#!/usr/bin/python
# -*- coding: utf-8 -*-

# 2017-08-29 added 8-bit CRC calculation as descripted in tensioLINK - ASCII - RAW Spezifikation -V1.0.englisch.pdf
# 2017-11-09 works with pqBARLog4 v 1.5
# 2018-10-16 Modulus CRC calculation

import sys
import binascii,struct
import numpy as np
import re

non_decimal = re.compile(r'[^\d.-]+')

ASCIIchr={
   "<NUL>":"\x00",
   "<SOH>":"\x01",
   "<STX>":"\x02",
   "<ETX>":"\x03",
   "<EOT>":"\x04",
   "<ENQ>":"\x05",
   "<ACK>":"\x06",
   "<BEL>":"\x07",
   "<BS>":"\x08",
   "<TAB>":"\x09",
   "<LF>":"\x0A",
   "<BS>":"\x0B",
   "<BS>":"\x0C",
   "<CR>":"\x0D",
   "<SO>":"\x0E",
   "<SI>":"\x0F",
   "<DLE>":"\x10",
   "<DC1>":"\x11",
   "<DC2>":"\x12",
   "<DC3>":"\x13",
   "<DC4>":"\x14",
   "<NAK>":"\x15",
   "<SYN>":"\x16",
   "<ETB>":"\x17",
   "<CAN>":"\x18",
   "<EM>":"\x19",
   "<SUB>":"\x1A",
   "<ESC>":"\x1B",
   "<FS>":"\x1C",
   "<GS>":"\x1D",
   "<RS>":"\x1E",
   "<US>":"\x1F",
   "<SPACE>":"\x20",
   "<DEL>":"\x7F"}

chrSize={
   'h':2,
   'H':2,
   'i':4,
   'I':4,
   'l':4,
   'L':4,
   'f':4,
   'q':8,
   'Q':8,
   'd':8}

anySpace={}

for i in range(256):
   ASCIIchr["<0x%02X>"%i]=chr(i)
for i in range(50):
   anySpace["<%iX>"%i]=i
def replaceASCIIlist(ls):
   for i,key in enumerate(ls):
      if key in ASCIIchr:
         ls[i]=ASCIIchr[key]
   return ls
def replaceASCIIstring(st):
   for key, value in ASCIIchr.iteritems():
      st=st.replace(key,value)
   return st
def strlToFloatl(strl):
   floatl=[]
   for s in strl:
      floatl.append(strToFloat(s))
   return floatl
def strToFloat(strf):
   try:
      ff=float(non_decimal.sub('',strf))
   except:
      ff=None
   return ff
def crcModulus(data, mod=256, endian=">"):
   if mod<=256:
      ctype=endian+"B"
   elif mod<=65536:
      ctype=endian+"H"
   elif mod<=4294967296:
      ctype=endian+"I"
   crc=struct.pack(ctype, sum(bytearray(data))%mod)
   return crc
def crc16(data, bits=8):
  data=data.encode("hex")
  crc = 0xFFFF
  for op, code in zip(data[0::2], data[1::2]):
    crc = crc ^ int(op+code, 16)
    for bit in range(0, bits):
      if (crc&0x0001)  == 0x0001:
        crc = ((crc >> 1) ^ 0xA001)
      else:
        crc = crc >> 1
  msb = format((crc >> 8), '02x')
  lsb = format((crc & 0x00FF), '02x')
  #print lsb
  #print msb
  return (lsb+msb).decode("hex")
def crc8(data):
   # Checksum consists of 1 byte and is generated by overflow over all data bytes as the 8 bit sum.
   bit8sum=np.sum(bytearray(data),dtype=np.uint8)
   # The sum of all bytes + checksum = 0.
   zerosum=np.uint8(256-bit8sum)
   zerosumhex=chr(zerosum)
   return zerosumhex
def idecode(dalist,freg,ctype):
   lreg=freg+chrSize[ctype.translate(None, "<>!=*")]
   dalist=dalist[freg:lreg]
   if "*" in ctype:
      ctype=ctype.replace("*","")
      dalist=[''.join(x) for x in zip(dalist[0::2],dalist[1::2])]
      dalist.reverse()
   decoded = struct.unpack(ctype, binascii.unhexlify("".join(dalist)))
   return decoded
def iencode(number, ctype):
   if "*" in ctype:
      ctype=ctype.replace("*","")
      a16little=True
   else: a16little=False
   encoded = struct.pack(ctype, number).encode('hex')
   if a16little:
      encoded=[encoded[i:i+4] for i in range(0, len(encoded), 4)]
      encoded.reverse()
      encoded="".join(encoded)
   encoded=[encoded[i:i+2] for i in range(0, len(encoded), 2)]
   return encoded
def main():
   print strlToFloatl(["000"])
   while True:
      hexstring = raw_input("Hex string: ").replace(" ", "")
      if hexstring == 'q'or hexstring == 'Q':
         break
      try:
         crc=crc16(hexstring.decode("hex"))
      except:
         print "Hex string error"
      else:
         print crc.encode("hex")
   sys.exit()

if __name__ == '__main__':
   main()
