"""
   Common/general python routines (libraries).
   
   Revision history:
      2006-09-07  FormatExceptionMessage, shortform added
"""
import os,sys,traceback

class epylib_Error(Exception):
   """
      Common exception class
   """
   def __init__(self,value):
      self.value=value
   def __str__(self):
      return repr(self.value)


def FormatExceptionMessage(strng='',shortform=True):
   """
      Format the last occurred exception as a one-line message
      with optional prefix (strng). Return the formatted string.
      
      Revision history:
         2006-11-24  bug fix in case of shortform not founding "Exception"
   """
   try:
      s=strng+''.join(traceback.format_exception(
         *sys.exc_info())[-2:]).strip().replace('\n',': ')      
   except:
      s="ERROR in FormatExceptionMessage!!!!"
   rf=0
   if shortform:
      rf=s.rfind('Exception:')
      if rf>0:
         while rf>=0 and s[rf]!=' ': rf-=1
         rf+=1
      else:
         rf=0
   return s[rf:]
   
  
  
try:
   import termios,fcntl  # not in windows!
   
   class TermIO:
      """
         Terminal (one character at a time without blocking) input
      """
      def __init__(self,start=True):
         '''
            Initialise terminal mode (if start==True)
         '''
         self.started=False
         self.fd=None
         self.oldterm=None
         self.oldflags=None
         if start: self.start()
         
      def __del__(self):
         '''
            stop it
         '''
         self.stop()
         
           
      def start(self):
         ''''
            Initialise terminal for one character at a time input
            (linux). Return file descriptor (fd).
         '''
         if self.started:
            self.stop()
         self.fd=sys.stdin.fileno()
         # Set terminal (saving old settings)
         self.oldterm=termios.tcgetattr(self.fd)
         newattr=termios.tcgetattr(self.fd)
         newattr[3]=newattr[3] & ~termios.ICANON & ~termios.ECHO
         termios.tcsetattr(self.fd,termios.TCSANOW,newattr)
         self.oldflags=fcntl.fcntl(self.fd,fcntl.F_GETFL)
         fcntl.fcntl(self.fd,fcntl.F_SETFL,self.oldflags | os.O_NONBLOCK)
         self.started=True
         return self.fd
      
      
      def stop(self):
         '''
            End the terminal mode. Return None
         '''
         if not self.started: return None
         termios.tcsetattr(self.fd,termios.TCSAFLUSH,self.oldterm)
         fcntl.fcntl(self.fd,fcntl.F_SETFL,self.oldflags)
         self.started=False
         return None
       
       
      def GetKeybChar(self):
         '''
            Return LAST character in keyboard buffer, or None
            if nothing found.
         '''
         charin=None
         try:
            while True:    # read until no more characters found (==execption)
               ch=sys.stdin.read(1)
               charin=ch.upper()
         except IOError:
            pass
         return charin
except:
   try:
      from msvcrt import getch,kbhit
         
      class TermIO:
         def __init__(self,start=True):
            pass
         def __del__(self):
            pass
         def start(self):
            pass
         def stop(self):
            pass
         def GetKeybChar(self):
            '''
            '''
            c=None
            while kbhit():
               c=getch()
               c=c.upper()
            return c
   except:
      pass
      


