# -*- coding: utf-8 -*-
"""
   Routines for accessing db "lasers" tables meta_instruments
   

   Revision history:
      2017-02-16  First version
      2017-09-19  station 'pal' added
      2018-02-24  manual additions
"""
from jahdblib import cDB
import mx.DateTime as DT

DTDT=DT.DateTime

DBNAME='lasers'
METATABLENAME='meta_instruments'


TABLEDEFS={
   METATABLENAME:
   """
      instrument_id        TEXT,
      install_utc          TIMESTAMP WITHOUT TIME ZONE,
      removal_utc          TIMESTAMP WITHOUT TIME ZONE,
      instrument_type      TEXT,
      station              TEXT,
      note                 TEXT,
      PRIMARY KEY(instrument_id,install_utc)
   """
}

INSTRTYPES=['pic','lgr','gc','lic']
STATIONS=['sam','kum','uto','sod','pui','pal']

def GetInstrumentsAtStation(station,instrtype=None,startdt=None,lastdt=None,instrid=None):
   '''
      Return [[instrument_id,installdt,removaldt,instrument_type,station,comment]] in install_utc order
      
      If instrtype or instrid is given, only fitting instrument are returned
   '''
   callstr="instruments.GetInstrumentsAtStation(%s,%s,%s,%s,%s):"%(station,instrtype,startdt,lastdt,instrid)
   if station not in STATIONS or instrtype and instrtype not in INSTRTYPES:
      print 
      print callstr
      if station not in STATIONS:
         print "\nInstrument type must be in %s"%(INSTRTYPES,)
      else:
         print "\nstation must be in %s"%(STATIONS,)
      print
      raise ValueError("instruments.GetInstrumentsAtStation()")
   db=cDB(DBNAME)
   cu=db.cursor()
   whs="AND instrument_type='%s'"%(instrtype,) if instrtype else ''
   if instrid:
      whs+=" AND instrument_id='%s'"%(instrid,)
   sq="SELECT * FROM \"%s\" WHERE station='%s' %s "\
      " ORDER BY install_utc;"%(METATABLENAME,station,whs)
   cu.execute(sq)
   da=cu.fetchall()
   cu.close()
   ret=[]
   if startdt or lastdt:
      for t in da:
         if startdt and t[2] and t[2]<=startdt: continue      # too early
         if lastdt and t[1]>=lastdt: break
         ret.append(list(t))
   else:
      ret=[list(t) for t in da]
   if not ret:
      print 
      print callstr
      print
      print "No instrument(s) found!\n"
   if station=='sam':
      if instrid=='cfads098' and len(ret)==2:   # combine
         ret=[[ret[0][0],ret[0][1],DTDT(2012,10,12),ret[0][3],ret[0][4],ret[0][5]]]
      elif instrid=='cfkads2018':
         ret=[[ret[0][0],ret[0][1],DTDT(2012,6,12,10),ret[0][3],ret[0][4],ret[0][5]]]
   return ret

      
   

################################################################################      

def main():
   tbl=METATABLENAME
   print "\nCalled directly only creates table %s, without dropping old one(s)"%(tbl,)
   print "\nEnable by editing main()"
   if True:
      return
   else:
      import mx.DateTime as DT
      DTDT=DT.DateTime
      instr=GetInstrumentsAtStation('sam','pic',DTDT(2016,5,1),)
      print "\nResults:"
      for i in instr: 
         print i
      return
   if False:
      import mx.DateTime as DT
      DH=DT.TimeDelta
      db=cDB('results')
      print "Copying data from results/meta_gases_hmeans"
      cu=db.cursor()
      cu.execute("SELECT * FROM meta_gases_hmeans WHERE component='CO2' OR instrument='gc' ORDER BY station,start_dt;")
      da=cu.fetchall()
      cu.close()
      db.close()
      db=cDB(DBNAME)
      cu=db.cursor()
      for d in da:
         if d[3]=='pic':
            iid=d[-2].split('_')[1]
         else:
            iid=d[3]
         sq="INSERT INTO %s VALUES('%s','%s','%s','%s','%s',NULL);"%(
            tbl,iid,d[4],d[5],d[3],d[0])
         sq=sq.replace('None','NULL')
         sq=sq.replace("'NULL'","NULL")
         print sq
         cu.execute(sq)
      db.commit()
      return
            
   else:
      db=cDB(DBNAME)
      print "Creating table %s ..."%(tbl,)
      try:
         db.CreateTable(tbl,TABLEDEFS[tbl]) #,drop_existing=True)
      except Exception as e:
         db.rollback()
         print "Exception - table already exists...(?):"
         print e

if __name__ == "__main__":
   main()
