# -*- coding: latin-1 -*-
"""
   Common/general python routines (libraries).
   
   Revision history:
      2007-10-08/09 first version
      2008-02-22 updates
      2008-08-27 CalcWSRes +'linodrfit'
      2008-09-05 Also 102-105 allowed as "cal_ports" in table cal_times
"""
import os
import numpy
import mx.DateTime as DateTime
import quad_odr as odr
import jahdblib,jahplotlib

DBNAME="fmico2"
DTDT=DateTime.DateTime

LI7000INSTALL=DTDT(2006,7,8,17)
NEWSYSTEMSTART=DTDT(2011,8,1)

def CalcPlotStdConc(calstart,calset,figdir='.',excluded=[],version=None,data=None):
   '''
      Calculate and plot concentration for each station (calibration)
      standard based on other standards excluding cal. cycles with
      indexes in excluded[].
      
      Plot figures to dir figdir, and return
      {port:{'id':'', 'version':n, 'fig':'with dir', 'excluded':[],
             'conc': xxx.xx, 'dev': x.xx, 'assigned': c.cc, 'comment':'txt',
             'calstart':dt,'nvals':n,'calset':v}
      }
   '''
   print "CalcPlotStdConc(%s,%s,data?:%s)"%(calstart,calset,data!={})
   ret={}
   calports=[]
   db=jahdblib.cDB(DBNAME)
   cu=db.cursor()
   if data:
      ret=data
      calports=data.keys()
      calstart=data[calports[0]]['calstart']
      if not excluded:
         excluded=data[calports[0]]['excluded']
   figfile=os.path.join(figdir,'stdconc_%10.10s.png'%calstart)
   
   if not data:
      sqls="SELECT * FROM cal_times WHERE cal_start>='%s' AND cal_set=%s ORDER BY cal_start LIMIT 1;"%(
         calstart,calset)
      cu.execute(sqls)
      caldata=cu.fetchall()
      if not caldata[0]:
         cu.close() ; db.close() ; return ret
      
      for cp in caldata[0][2]: 
         calports.append(cp)
         ret[cp]={}
         ret[cp]['comment']=caldata[0][3]
         ret[cp]['calstart']=caldata[0][0]
         ret[cp]['calset']=int(calset)
      del caldata
   cu.close(); db.close()
      
   stdconc=GetCalStdConc(calstart,calports,version)
   if not data:
      for cp in calports:
         try:
            ret[cp]['id']=stdconc[cp]['id']
            ret[cp]['assigned']=stdconc[cp]['conc']
            ret[cp]['version']=stdconc[cp]['version']
         except:
            print "\n!!!!!!!!   EXCEPTION   !!!!!!"
            print "key:",cp
            print ret[cp]
            print "stdconc[key]:"
            print stdconc[cp]
            raise

   mdata=GetMeasData(calstart,calports)
      
   plotdata=[[],[],[],[]]
   for p in calports:   # through every port
      x=[] ; sdx=[] ; conc=[] ; sdconc=[]
      plotdata[0].append(p)
      yd=[] ; syd=[]
      for cy,da in enumerate(mdata):
         dt=da[0] ; pda=[]
         if cy+1 in excluded: continue
         if p==calports[0]: plotdata[1].append(cy+1)
         for d in da[1]:
            if d[0]==p:
               pda=d
            else:
               x.append(d[1])
               sdx.append(d[2])
               conc.append(stdconc[d[0]]['conc'])
               sdconc.append(stdconc[d[0]]['dev'])
         try:
            r,res=odr.Fit(x,conc,sdx,sdconc)
         except:
            print x ; print conc
            print sdx ; print sdconc
            raise
         try:
            cc=r[0]*pda[1]*pda[1]+r[1]*pda[1]+r[2]
            sdcc=r[0]*pda[2]*pda[2]+r[1]*pda[2]
         except:
            print "\n!!!!!!  EXCEPTION   !!!!!!!!"
            print "x,conc:",x,conc
            print "devx,devconc:",sdx,sdconc
            print "fit res:",r,res
            print "port:%s - port data:%s"%(p,pda)
            print
            raise
         yd.append(cc) ; syd.append(sdcc)
      plotdata[2].append(yd)
      plotdata[3].append(syd)
      ret[p]['conc']=numpy.average(yd)
      ret[p]['dev']=numpy.std(yd)
      ret[p]['nvals']=len(yd)
      ret[p]['excluded']=excluded
      ret[p]['fig']=figfile

      
   PlotStdConc(plotdata,figfile,"Calculated  Std  Conc,  %10.10s  (v  %s)"%(
      calstart,version))

   return ret
      
      
def GetCalStdConc(calstart,ports,version=1):
   '''
      Return {port:{'id':'cylid', 'version':version, 'conc':xxx.xx, 'dev':x.xx,
         'comment':'txt', 'install':dt, 'remove':dt, 'calibrated':dt}}
      
      If version==None, return data for latest found
   '''
   ret={}
   ps=ports
   if type(ports)==type(1): ps=[ports]
   db=jahdblib.cDB(DBNAME)
   cu=db.cursor()
   versions='' if version==None else "version=%s AND"%version
   sqls="SELECT * FROM cal_std_conc WHERE %s port IN (%s) "\
   " AND dt_install<='%s' AND dt_remove>'%s' ORDER BY version DESC;"%(versions,
      str(ports)[1:-1],calstart,calstart)
   cu.execute(sqls)
   data=cu.fetchall()
   cu.close() ;  db.close()   
   if not data: # see if we can find data with CO2WStdConc.WStdConc()
      from CO2WStdConc import WStdConc
      for p in ps:
         r=WStdConc(p,calstart)
         if r and r[0]>0:
            ret[p]={'comment':'WS!','conc':r[0],'dev':r[1],'id':p,
               'version':version}
      return ret
   
   v=data[0][1]
   for da in data: 
      if da[1]!=v: break
      p=da[2]
      ret[p]={}
      ret[p]['comment']=da[8]
      ret[p]['conc']=da[6]
      ret[p]['dev']=da[7]
      ret[p]['id']=da[0]
      ret[p]['version']=da[1]
      ret[p]['install']=da[3]
      ret[p]['remove']=da[4]
      ret[p]['calibrated']=da[5]
   
   return ret
   
   
def GetStdInternalCal(calstart,calset):
   '''
      Return from db 
      {port:{'excluded','conc','dev','calset','calstart','assigned',
      'comment','flag','nvals'}}
      or {} if no data in db
   '''
   ret={}
   db=jahdblib.cDB(DBNAME)
   cu=db.cursor()
   sqls="SELECT * FROM cal_std_from_std WHERE cal_start='%s' AND cal_set=%s;"%(
      calstart,calset)
   cu.execute(sqls)
   data=cu.fetchall()
   for d in data:
      elist=d[3]
      if type(elist)!=type([]): elist=[]
      ret[d[2]]={'calstart':d[0],'calset':d[1],'excluded':elist,
         'nvals':d[5],'conc':d[6],'dev':d[7],'assigned':d[8],'comment':d[9]}
   cu.close() ; db.close()
   return ret
   
def GetWSCal(calstart,stdver):
   '''
      Return from db 
      {port:{'excluded','conc','dev','stdver','calstart',
      'comment','flag','nvals','max_min','calcdate','wsid'}}
      or {} if no data in db
   '''
   ret={}
   db=jahdblib.cDB(DBNAME)
   cu=db.cursor()
   sqls="SELECT * FROM ws_meas_res WHERE cal_start='%s' AND std_version=%s;"%(
      calstart,stdver)
   cu.execute(sqls)
   data=cu.fetchall()
   cu.close() ; db.close()
   for d in data:
      elist=d[6]
      if type(elist)!=type([]): elist=[]
      ret[d[2]]={'calstart':d[0],'stdver':d[1],'excluded':elist,
         'nvals':d[5],'conc':d[7],'dev':d[8],'max_min':d[9],
         'flag':d[10],'comment':d[11],'calcdate':d[3],'wsid':d[4]}
   
   return ret
   
   
def SetStdInternalCal(data):
   '''
   '''
   db=jahdblib.cDB(DBNAME)
   cu=db.cursor()
   sqls="INSERT INTO cal_std_from_std VALUES('%s',%s,%s,'{%s}',%s,%s,%s,%s,%s,%s);"
   for p in data:
      d=data[p]
      try:
         flags="'%s'"%d['flag']
      except:
         flags='NULL'
      comments='NULL'
      if d['comment']: comments="'%s'"%d['comment']
      s=sqls%(d['calstart'],d['calset'],p,str(d['excluded'])[1:-1],flags,
         d['nvals'],d['conc'],d['dev'],d['assigned'],comments)
      try:
         cu.execute(s)
      except:
         db.rollback()
         cu.execute("DELETE FROM cal_std_from_std WHERE cal_start='%s' AND cal_set=%s AND port=%s;"%(
               d['calstart'],d['calset'],p))
         db.commit()
         cu.execute(s)
      db.commit()
   
   cu.close() ; db.close()
   
def SetWSCal(data):
   '''
   '''
   db=jahdblib.cDB(DBNAME)
   cu=db.cursor()
   sqls="INSERT INTO ws_meas_res VALUES('%s',%s,%s,'%s','%s',%s,'{%s}',%s,%s,%s,%s,%s);"
   for p in data:
      d=data[p]
      if not d.has_key('conc'): continue
      try:
         flags="'%s'"%d['flag']
      except:
         flags='NULL'
      if flags=="'None'": flags='NULL'
      comments='NULL'
      if d['comment']: comments="'%s'"%d['comment']
      s=sqls%(d['calstart'],d['stdver'],p,d['calcdate'],d['wsid'],d['nvals'],
         str(d['excluded'])[1:-1],d['concw'],d['devw'],
         d['max_min'],flags,comments)
      try:
         cu.execute(s)
      except:
         db.rollback()
         cu.execute("DELETE FROM ws_meas_res WHERE cal_start='%s' AND std_version=%s AND "\
            "calc_date='%s' AND port=%s;"%( d['calstart'],d['stdver'],d['calcdate'],p))
         db.commit()
         try:
            cu.execute(s)
         except:
            print ; print s
            raise
      db.commit()  
   cu.close() ; db.close()
   
def GetMeasData(calstart,ports=None):
   '''
      Return measured data for all ports by default (i.e.
      ports==None) for calibration starting at
      calstart:
      
      [cycle_start,[[port],[y],[dev]]]
      
      Revision history:
         2008-12-22 aflag is in (NULL,' ','.') removed
   '''
   csdt=sdt=calstart
   if type(csdt)==type(''):
      csdt=DateTime.ISO.ParseDateTime(csdt)
   try:
      sdt=DateTime.ISO.ParseDateTime(calstart)
   except:
      pass
   if sdt>=NEWSYSTEMSTART:
      sq="li_co2,li_co2_dev FROM sammal_co2_1min_raw"
   elif sdt>=LI7000INSTALL:
      sq="mean_co2_ppm,dev_co2_ppm FROM sammal_co2raw7000"
   else:
      sq="mean_co2_mv,dev_co2_mv FROM sammal_co2raw"
   
   ret=[] ; sports=""
   if ports: 
      sports="AND port IN (%s)"%str(ports)[1:-1]
      
   db=jahdblib.cDB(DBNAME)
   cu=db.cursor()
   sqls="SELECT datetime,port,%s WHERE "\
   "datetime>='%s' AND datetime<'%s' %s AND port NOT IN (1,101) " \
   "AND adjusting='f' ORDER BY datetime;"%(sq,sdt,sdt+1,sports)

   cu.execute(sqls)
   print sqls
   print "rowcount:",cu.rowcount
   data=cu.fetchall()
   for d in data:
      print d[0],d[1:]
   cu.close() ;  db.close()
   if not data: return ret
   
   cylen=0
   daports=[data[0][1]] ; daps={}
   for i,da in enumerate(data[1:]):
      if cylen==0 and (da[1]==data[0][1] or da[1]==data[i][1]):
         cylen=i+1
         break
      daports.append(da[1])
      
   print "daports",daports
   plen=len(daports)
   for i in range(plen):
      p=daports[i]
      if i==0:
         daps[p]=[p,daports[i+1],daports[plen-1]]
      elif i==plen-1:
         daps[p]=[p,daports[0],daports[i-1]]
      else:
         daps[p]=[daports[i-1],daports[i+1]]
   
   r=[] ; cs=None ; pdt=None
   for i,da in enumerate(data):
      if not cs: cs=da[0]
      if i>0:
         try:
            if da[1] not in daps[data[i-1][1]]: 
               print "port %s (%s) not in daps (%s) at %s"%(
                  da[1],data[i-1][1],daps,da[0])
               print "\n! discontinuing !\n"
               break
         except:
            raise
            pass
      if pdt and (da[0]-pdt).minutes>40: 
         print "too long gap:",da[0]
         break
      r.append([da[1],da[2],da[3]])
      if (i+1) % cylen == 0:
         ret.append([cs,r])
         cs=None
         r=[]
      pdt=da[0]
   return ret

def PlotStdConc(data,outfile,title):
   '''
      Plot calculated WS concentrations for each cycle along
      with stdev based on measurement result deviation (not on
      how well the calibr. standard fit was)
      
      data:
      [[ports][x][[y]][[stdy]]]
   '''
   
   fd={'x':data[1],'kwargs':[],
      'y':[],'ylabel':[],'grid':'y','yerr':[]}
   for da in data[0]:
      fd['ylabel'].append('%s'%da)
      fd['kwargs'].append({'ms':5,'ls':'-','mfc':'white'})
      
   fd['y']=data[2]
   fd['yerr']=data[3]
      
   fd['title']=(title,'fontsize=14')
   fd['frame']={'top':.94,'right':.97,'bottom':.05,'left':.1}
   cp=jahplotlib.cPlotXY(size="750,600")
   cp.PlotErrorbar(fd,outfile,onecol=True)
   
def PlotCalStdConcTimeser(figdir):
   '''
      return fig basenames
   '''
   ret=[]
   db=jahdblib.cDB(DBNAME)
   cu=db.cursor()
   for calset in range(1,3):
      for pset in range(2):
         ports=''
         if calset==1:
            if pset==0:
               ports="AND port<13"
            else:
               ports="AND port>200"
         else:
            if pset>0: continue
         
         sqls="SELECT cal_start,port,nvals,conc,dev,assigned FROM cal_std_from_std"\
        " WHERE cal_set=%s %s ORDER BY cal_start;"%(calset,ports)
         cu.execute(sqls)
         data=cu.fetchall()
         
         fd={'kwargs':[],'ylabel':[],'grid':'x','x':[],'y':[],'yerr':[],'lineat':[]}
         x={} ; y={} ; ye={} ; k=[]
         for d in data:
            if not d[1] in k:
               k.append(d[1])
               x[d[1]]=[]
               y[d[1]]=[]
               ye[d[1]]=[]
               fd['ylabel'].append('%s'%d[1])
               fd['kwargs'].append({'ms':5,'ls':'-','mfc':'white'})
               fd['lineat'].append(d[5])
            x[d[1]].append(d[0].absdays+1)
            y[d[1]].append(d[3])
            ye[d[1]].append(d[4])
         
         for d in k:
            fd['x'].append(x[d]) 
            fd['y'].append(y[d])
            fd['yerr'].append(ye[d])
         fd['title']=("Calibration  Standard  Set  %s"%calset,'fontsize=14')
         fd['frame']={'top':.95,'right':.97,'bottom':.05,'left':.08}
         fd['xdate']=True
         cp=jahplotlib.cPlotXY(size="900,700")
         ret.append('calset%s_%s.png'%(calset,pset))
         outfile=os.path.join(figdir,ret[-1])
         cp.PlotErrorbar(fd,outfile,onecol=True)
      
   cu.close() ; db.close()
      
   return ret
   

def CalcPlotWSConc(calstart,stdver,figdir='.',excluded=[],cal_set=1,data=None):
   '''
      Calculate and plot WS concentrations excluding cal. cycles with
      indexes in excluded[].
      
      Plot figures to dir figdir, and return
      {port:{'version':n, 'fig':'with dir', 'excluded':[],
             'conc': xxx.xx, 'dev': x.xx, 'comment':'txt',
             'calstart':dt,'nvals':n,'calset':v, 'max_min':v,
             'calcdate':NOW(),'wsid':''}
      }
   '''
   print "CalcPlotWSConc(%s,stdver:%s,data?:%s)"%(calstart,stdver,data!={})
   ret={}
   db=jahdblib.cDB(DBNAME)
   cu=db.cursor()
   if data:
      ret=data
      calports=data.keys()
      calstart=data[calports[0]]['calstart']
      if not excluded:
         excluded=data[calports[0]]['excluded']
   figfile=os.path.join(figdir,'wsconc_%10.10s_%s.png'%(calstart,stdver))

   sqls="SELECT * FROM cal_times WHERE cal_start>='%s' AND cal_set=%s ORDER BY cal_start LIMIT 1;"%(
      calstart,cal_set)
   print sqls
   cu.execute(sqls)
   caldata=cu.fetchall()
   print caldata
   if not caldata[0]:
      cu.close() ; db.close() ; return ret
   calports=[]
   calibrstart=None
   for cp in caldata[0][2]: 
      calports.append(cp)
      ret[cp]={}
      ret[cp]['comment']=caldata[0][3]
      ret[cp]['calstart']=caldata[0][0]
      if not calibrstart: calibrstart=caldata[0][0]
      ret[cp]['calset']=int(cal_set)
   del caldata
      
   print "\ncalibr start:",calstart
   print "calibr ports:",calports
   print "calibr std ver:",stdver
   print
   
   stdconc=GetCalStdConc(calstart,calports,stdver)
   if not data:
      for cp in calports:
         ret[cp]['id']=stdconc[cp]['id']
         ret[cp]['assigned']=stdconc[cp]['conc']
         ret[cp]['version']=stdconc[cp]['version']
         ret[cp]['stdver']=stdver

   mdata=GetMeasData(calstart)
#   print "mdata:",mdata
   mports=[]
   for m in mdata[0][1]: 
      if m[0] not in calports: 
         mports.append(m[0])
         sqls="SELECT ws_id FROM ws_cylinders WHERE port=%s AND dt_install<='%s' "\
            "AND dt_end>='%s'"%(m[0],calstart,calstart)
         cu.execute(sqls)
         if cu.rowcount==0:
            if m[0]>100:
               sqls="SELECT ws_id FROM ws_cylinders WHERE port=%s AND dt_install<='%s' "\
               "AND dt_end>='%s'"%(m[0]%100,calstart,calstart)
               cu.execute(sqls)
               if cu.rowcount>0:
                  wsid=cu.fetchone()[0]
               else:
                  wsid="N/A"
         else:
            wsid=cu.fetchone()[0]
         try:
            ret[m[0]]['wsid']=wsid
         except:
            ret[m[0]]={'wsid':wsid}
   cu.close() ; db.close()
   
   Fit=[] ; included=[]
   allFits=[] ; stdConcs=[] ; stdResiduals=[]
   for i,m in enumerate(mdata):
      x=[]; sdx=[]; conc=[]; sdconc=[] ; cps=[]
      for d in m[1]:
         if d[0] not in calports: continue
         cps.append(d[0])
         x.append(d[1]) ; sdx.append(d[2])
         conc.append(stdconc[d[0]]['conc'])
         sdconc.append(stdconc[d[0]]['dev'])
      allFits.append(odr.Fit(x,conc,sdx,sdconc))
      stdConcs.append(conc)
      sdx=[]
      fit=allFits[-1][0]
      for xx,sc in zip(x,conc):
         sdx.append(fit[0]*xx*xx+fit[1]*xx+fit[2]-sc)
      stdResiduals.append([cps,sdx])
      if i+1 in excluded: continue
      included.append(i+1)
      Fit.append(allFits[-1])
      
   figf=os.path.join(figdir,'ws_resvar_%10.10s_%s.png'%(calstart,stdver))
   PlotResVar(figf,allFits,"Residual  variance,  %10.10s  (stdver  %s)"%(calibrstart,stdver))
   figf2=os.path.join(figdir,"ws_fit_residuals_%10.10s_%s.png"%(calstart,stdver))
   PlotFitResiduals(figf2,stdConcs,stdResiduals,
      "Fit  Residuals,  %10.10s  (stdver  %s)"%(calibrstart,stdver))
   for p in calports: ret[p]['figs']=[figf2,figf]
   for p in mports: ret[p]['figs']=[figf2,figf]
   
   pdata={}
   for p in mports: pdata[p]=[[],[]]
   j=0
   for i,m in enumerate(mdata):
      if i+1 in excluded: continue
      x=[]; sdx=[]
      for d in m[1]:
         if d[0] in calports: continue
         r=Fit[j][0]
         cc=r[0]*d[1]*d[1]+r[1]*d[1]+r[2]
         sdcc=r[0]*d[2]*d[2]+r[1]*d[2]
         pdata[d[0]][0].append(cc)
         pdata[d[0]][1].append(sdcc)
      j+=1
         
   for p in mports:
      yd=pdata[p][0]
      ret[p]['conc']=numpy.average(yd)
      ret[p]['concw'],ret[p]['devw']=_WeightedAverage(pdata[p])
      ret[p]['dev']=numpy.std(yd)
      ret[p]['max_min']=max(yd)-min(yd)
      ret[p]['calcdate']="%10.10s"%DateTime.now()
      ret[p]['nvals']=len(yd)
      ret[p]['excluded']=excluded
      ret[p]['figs'].append(figfile)
      ret[p]['pdata']=pdata[p]
      ret[p]['included']=included
      ret[p]['calstart']=calibrstart
      ret[p]['stdver']=stdver
      if not ret[p].has_key('comment'): ret[p]['comment']=''
   for p in calports:
      ret[p]['figs'].append(figfile)
      
   PlotWSConc(ret,figfile,"WS/T  Concentrations,  %10.10s  (stdver  %s)"%(
      calstart,stdver))
   return ret
   
def _WeightedAverage(data):
   '''
      Return weighted average, weighted stdev (NOT unbiased!)
   '''
   da=data[0]
   sd=[x if x>.01 else .01 for x in data[1]]
   we=[1./x/x for x in sd]
   sumwe=sum(we)
   d=[x*w for x,w in zip(da,we)]
   wavg=sum(d)/sumwe
   
   i=0.
   for d,w in zip(da,we):
      x=d-wavg
      i+=w*x*x
   return wavg,numpy.sqrt(i/sumwe)
   
def PlotFitResiduals(outfile,stdconcs,stdresid,title):
   '''
   '''
   po=[[c,p] for c,p in zip(stdconcs[0],stdresid[0][0])]
   po.sort()
   ports=[p for x,p in po]
   conc={} ; leg=[] ; kwa=[]
   for c,p in po: 
      conc[p]=c
      leg.append('%s  (%.2f)'%(p,c))
      kwa.append({'lw':1.5,'ms':1.5})
   
   stds=stdconcs[0]
   stds.sort()
   fd={'y':[[] for i in range(len(ports))],'kwargs':kwa,'x':[i+1 for i in range(len(stdresid))],
      'grid':'y','axhline':'c="gray"','legend':leg}
   for por in stdresid:
      for p,r in zip(por[0],por[1]):
         iy=ports.index(p)
         fd['y'][iy].append(r)

   fd['title']=(title,'fontsize=11')
   fd['frame']={'top':.9,'right':.97,'bottom':.07,'left':.1}
   fd['xminmax']=[1,len(stdresid)]
   fd['ylabel']=[("Residuals  (ppm)","fontsize=11")]
   fd['legendfontsize']=8

   cp=jahplotlib.cPlotXY(size="750,220")
   cp.PlotXnY(fd,outfile)
   

def PlotResVar(outfile,fits,title):
   '''
   '''
   fd={'x':[],'kwargs':{'ms':4,'ls':'-','mec':'darkblue','mfc':'darkblue','marker':'o'}
      ,'y':[],'grid':'y'}
   y=[]
   for i,fit in enumerate(fits):
      fd['x'].append(i+1)
      y.append(fit[1])
   fd['y']=[y]
   fd['title']=(title,'fontsize=11')
   fd['frame']={'top':.83,'right':.97,'bottom':.1,'left':.1}
   fd['xminmax']=[1,len(y)]
   
   cp=jahplotlib.cPlotXY(size="750,130")
   cp.PlotXnY(fd,outfile)
   del cp
   
   
def PlotWSConc(ret,outfile,title):
   '''
      Plot calculated WS concentrations for each cycle along
      with stdev based on measurement result deviation (not on
      how well the calibr. standard fit was)
   '''
   
   fd={'x':[],'kwargs':[],'y':[],'ylabel':[],'grid':'y','yerr':[]}
   ports=[]
   for p in ret:
      try:
         if ret[p].has_key('pdata'): 
            ports.append(p)
            fd['ylabel'].append('%s'%p)
            fd['kwargs'].append({'ms':4,'ls':'-','mfc':'darkblue','mec':'darkblue','marker':'o'})
      except:
         pass
      
   x=[] ; y=[] ; yerr=[]

   for p in ports:
      y.append(ret[p]['pdata'][0])
      yerr.append(ret[p]['pdata'][1])
   
   fd['x']=ret[ports[0]]['included']
   fd['y']=y
   fd['yerr']=yerr
      
   fd['title']=(title,'fontsize=11')
   fd['frame']={'top':.94,'right':.97,'bottom':.04,'left':.1}
   fd['xminmax']=[fd['x'][0],fd['x'][-1]]
   cp=jahplotlib.cPlotXY(size="750,400")
   cp.PlotErrorbar(fd,outfile,onecol=True)

def PlotWSTimeSer(wsid,data,figdir,stdver="3"):
   '''
      return fig basenames
   '''
   ret=[]
   fd={'grid':'xy','x':[],'y':[],'yerr':[]}
   x=[] ; y=[] ; ye=[] ; yfit=[] ; yfite=[]
   fd['kwargs']=[{'ms':5,'ls':'-','mfc':'blue','marker':'o'}]
   fit=CalcWSRes(-1,data)['linodrfit']
   for d in data:
      x.append(d[0].absdays+1)
      y.append(d[1])
      ye.append(d[2])
      yfit.append(d[0].absdays*fit[0][0]+fit[0][1])
      yfite.append(0)
   fd['x']=x
   fd['y'].append(y)
   fd['y'].append(yfit)
   fd['yerr'].append(ye)
   fd['yerr'].append(yfite)
      
   fd['title']=("WS  %s  Calibrations,  stdver  %s"%(wsid,stdver),'fontsize=13')
   fd['frame']={'top':.90,'right':.97,'bottom':.07,'left':.08}
   fd['xdate']=True
   cp=jahplotlib.cPlotXY(size="800,280")
   ret.append('WS_%s.png'%(wsid,))
   outfile=os.path.join(figdir,ret[-1])
   cp.PlotErrorbar(fd,outfile,onefig=True)
   return ret
 
 
def CalcWSRes(wsid,data):
   '''
      Doesn't use par wsid!?
      
      Return
         {'conc':,'dev':,'wconc':,'wdev':,'linodrfit':}
         
      Revision history:
         2008-08-27 return also 'linodrfit':
         2008-08-29 'startconc','endconc'
         
   '''
   ret={}
   conc=[d[1] for d in data]
   ret['conc']=numpy.average(conc)
   ret['dev']=numpy.std(conc)
   
   sum=0. ; wsum=0. ; conc=[]
   for d in data:
      if d[4] not in (None,'.',' '): continue
      w=1./d[2]**2
      sum+=w*d[1]
      wsum+=w
   avg=sum/wsum
   
   sumr2=0. ; n=0
   for d in data:
      if d[4] not in (None,'.',' '): continue
      sumr2+=(avg-d[1])**2
      n+=1
   if n>1: n-=1
   ret['wconc']=avg
   ret['wdev']=numpy.sqrt(sumr2/n)
   
   x=[] ; xd=[] ; y=[] ; yd=[] ; dates=[]
   for d in data:
      if d[4] not in (None,'.',' '): continue
      dates.append(d[0].absdays)
      x.append(dates[-1])
      xd.append(1e-4)
      y.append(d[1])
      yd.append(d[2])
   fit=odr.Fit(x,y,xd,yd,model='unilinear')
   ret['linodrfit']=fit
   ret['startconc']=fit[0][0]*dates[0]+fit[0][1]
   ret['endconc']=fit[0][0]*dates[-1]+fit[0][1]
   
   return ret
