'''
   PostgreSQL handling routines

   Revision history:
      2010-09-22  by default CreateTable creates tables without OIDS
      2012-11-05  quote table names in CreateTable (-> case sensitivity)
      2012-11-08  filterwarnings
      2014-02-19  +GetColumnNames()
      2016-07-14  CreateTable: + index=
'''
import os,warnings
import psycopg2

warnings.filterwarnings('ignore','.*tempnam is a potential.*')

class cDB:
   '''
   Different db handling functions
   '''
   ORDER={True:'DESC',False:'ASC'}

   def __init__(self,dbname,host="hatakkaj.fmi.fi",
      user=None,password=None,default_seluser='psqluser'):
      '''
      default_seluser: leave empty, if no "GRANT SELECT"
      wanted for a user (when creating db).

      Revision history:
         2007-04-25  +host,user,password
      '''
      self.dbname=dbname
      self.host=host
      self.user=user
      self.password=password
      hosts=''
      if host: hosts="host=%s"%host
      if user and password:
         hosts+=" user=%s password=%s"%(user,password)
      self.dbh=psycopg2.connect("dbname=%s %s"%(self.dbname,hosts))
      self.seluser=default_seluser

   def cursor(self):
      return self.dbh.cursor()

   def close(self):
      self.dbh.close()

   def db(self):
      return self.dbh

   def rollback(self):
      self.dbh.rollback()

   def commit(self):
      self.dbh.commit()

   def SetField(self,tblname,field,wherefieldname,wherelist):
      '''
         for wh in wherelist:
            UPDATE tblname SET field WHERE wherefieldname=wh[0];
      '''
      if len(wherelist)==0: return 0
      try:
         v=wherelist[0][0]
      except:
         return 0
      n=0
      num=isinstance(v,type(1)) or isinstance(v,type(1.))
      cu=self.dbh.cursor()
      sqls="UPDATE %s SET %s WHERE %s=%%s;"%(tblname,field,wherefieldname)
      for wh in wherelist:
         if num:
            s=sqls%wh[0]
         else:
            s=sqls%"'%s'"%wh[0]
         cu.execute(s)
         n+=cu.rowcount
         self.dbh.commit()
      cu.close()
      return n


   def CreateTable(self,tblname,tbldef,drop_existing=False,verbose=False,withoids=False,index=None):
      """
      Create a table optionally dropping existing.
      tbldef contains table definition as a (multiline) string
      
      PosgreSQL should drop index automatically when dropping table...?
      """
      cu=self.dbh.cursor()
      if withoids:
         oids="WITH OIDS"
      else:
         oids="WITHOUT OIDS"
      sqls='''CREATE TABLE "%s" (%s) %s;''' %(tblname,tbldef,oids)
      if verbose:
         print "\n%s (drop_existing: %s)"%(sqls,drop_existing)

      if drop_existing:
         try:
            cu.execute('''DROP TABLE "%s";'''%tblname)
         except:
            self.dbh.rollback()
         else:
            self.dbh.commit()
            
      try:
         cu.execute(sqls)
      except:
         print
         print sqls
         print
         raise
         
      if index:
         idxcols=[index] if type(index)!=type([]) else index
         for idxcol in idxcols:
            sqls="CREATE INDEX ON %s (%s);"%(tblname,idxcol)
            if verbose:
               print "  %s"%(sqls,)
            try:
               cu.execute(sqls)
            except:
               print
               print sqls
               print
               raise
            self.dbh.commit()

      if self.seluser:
         cu.execute('''GRANT SELECT ON "%s" TO psqluser;'''%tblname)
         self.dbh.commit()
      cu.close()

   def Insert(self,tblname,valuestr,replacecond=None,nodo=False):
      '''
         Insert values in string valuestr (formatted for SQL
         INSERT) in tblname.

         if replacecond!=None, replaces existing data (i.e. INSERT,
         and if no luck does DELETE -> INSERT)

         Revision history:
            2006-08-02 replacecond,nodo added
      '''
      sqls='INSERT INTO "%s" VALUES(%s);'%(tblname,valuestr)
      if nodo:
         print sqls
         return
      cu=self.dbh.cursor()
      try:
         cu.execute(sqls)
      except psycopg2.IntegrityError:
         self.dbh.rollback()
         if replacecond:
            sdel='DELETE FROM "%s" WHERE %s;'%(tblname,replacecond)
            cu.execute(sdel)
            self.dbh.commit()
            cu.execute(sqls)
         else:
            raise
      self.dbh.commit()
      cu.close()


   def CopyData2Table(self,tblname,data,dataidx=None,stridx=None,
      addconstcols=None,delimiter=',',nodo=False,constcolspos=None):
      '''
         Writes out the data (column numbers mentioned in dataidx in that order,
         with ''s around columns mentioned in stridx, i.e. non numeric)
         to a file and COPYies it to a table.

         Additionally extends data line (columns) by adding content of the
         contentconstantto the end of the normal data by default, OR
         to positions constcolspos[]

         if nodo==True: only prints out the sql string (i.e. no table update)
         and does not remove temporary data file [for debugging].

         Revision history
            2006-07-25  major changes: constcolspos, dataidx order
            2006-07-26  return number of rows inserted (.rowcount does not work)
            2006-07-31  bug fix (if constcolspos indexes are consecutive)
            2007-08-06  add backslash before \
            2010-02-23  postgresql 8.4: COPY format changed(?) - fixed
            2010-05-04  os.tempnam
            2012-11-28  None->''
      '''
      rows=0
      if not data: return rows
      flname=os.tempnam("/tmp")
      ofs=open(flname,'w')
      strok=True
      for da in data:
         vals=[]
         datafieldno=0
         if not dataidx: dataidx=xrange(len(da))
         for idx in dataidx:
            if constcolspos!=None and datafieldno in constcolspos:
               for ccpin,ccp in enumerate(constcolspos):
                  if ccp==datafieldno: vals.append(addconstcols[ccpin])
            v=da[idx]
            if stridx and idx in stridx:
               v2=[]
               v="%s"%v
               for i,s in enumerate(v):
                  v2.append(s)
                  if s=='\\':
                     try:
                        if v[i+1]!='\\': v2.append('\\')
                     except:
                        v2.append('\\')
               v=''.join(v2)
               v="'%s'"%v            
            if v==None: 
               v=''
            elif type(v)!=type(''):
               v="%s"%v
            vals.append(v)
            datafieldno+=1
         if addconstcols and constcolspos==None:
            vals.extend(addconstcols)
         if strok:
            try:
               ostr=','.join(vals)+'\n'
            except:
               print da
               print vals
               raise
#            ostr=ostr.replace('None','NULL').replace("'NULL'",'NULL')
            try:
               ofs.write(ostr)
            except TypeError:
               strok=False
         if not strok:
            ofs.write(delimiter.join([str(v) for v in vals])+'\n')
         rows+=1
      ofs.close()
      sqls='''COPY "%s" FROM '%s' DELIMITER '%s' NULL AS '';'''%(
            tblname,flname,delimiter)
      if not nodo:
         cu=self.dbh.cursor()
         try:
            cu.execute(sqls)
         except psycopg2.IntegrityError:
            print
            print sqls
            print
            self.dbh.rollback()
            raise
         except:
            print
            print sqls
            print
            raise
         self.dbh.commit()
         cu.close()
         os.unlink(flname)
      else:
         print "\nCopyData2Table: tmp file: %s, sql:\n%s"%(flname,sqls)
      return rows


   def CopyFromFile(self,tblname,filename,delimiter=',',rm_file=False):
      '''
         Use sql COPY x FROM x USING DELIMITERS ''...
         for file filename, optionally deleting it after COPY

         Revision history:
            2006-07-26  return number of inserted rows (DOES NOT work -> removed)
      '''
      cu=self.dbh.cursor()
      sqls='''COPY "%s" FROM '%s' USING DELIMITERS '%s' WITH NULL AS '';'''%(
            tblname,filename,delimiter)
      try:
         cu.execute(sqls)
      except psycopg2.IntegrityError:
         self.dbh.rollback()
         raise
      self.dbh.commit()
      cu.close()
      if rm_file: os.unlink(filename)


   def DeleteData(self,tblname,startdt,enddt,dtcolname="datetime",where=None,quiet=False):
      '''
         Delete data from tblname based on column datetime
      '''
      if not quiet:
         print "Deleting from %s, %19.19s <= %s < %19.19s"%(
            tblname,startdt,dtcolname,enddt)
      cu=self.dbh.cursor()
      wh=""
      if where: wh="AND %s"%where
      sqls='''DELETE FROM "%s" WHERE %s>='%s' AND %s<'%s' %s;'''%(
         tblname,dtcolname,startdt,dtcolname,enddt,wh)
      cu.execute(sqls)
      self.dbh.commit()
      cu.close()


   def GetDataDt(self,tblname,dtcolname="datetime",last=True,where=None,show=False):
      '''
         Return last/first datetime from tblname where
         SQL condition==where

         Revision history:
            2011-03-24 param. show added
      '''
      cu=self.dbh.cursor()
      wh=''
      if where: wh=" WHERE %s"%where
      sqls='''SELECT %s FROM "%s" %s ORDER BY %s %s LIMIT 1;'''%(
         dtcolname,tblname,wh,dtcolname,self.ORDER[last])
      try:
         if show:
            print 'cDB.GetDataDt: "%s"'%sqls
         cu.execute(sqls)
      except:
         print "Something wrong with:"
         print sqls
         raise
      da=cu.fetchone()
      cu.close()
      if da: return da[0]
      return None
      
   def GetColumnNames(self,tblname):
      '''
         Return list of column names in tblname
      '''
      cu=self.dbh.cursor()
      sqls='''SELECT column_name FROM information_schema.columns 
      WHERE table_name='%s';'''%(tblname,)
      cu.execute(sqls)
      da=cu.fetchall()
      return [x[0] for x in da]
      
      
