#-*- coding: latin-1 -*-
'''
   Plotting routines (using matplotlib)

   Devel start 2006-03-15

   Revision history:
      2007-02-16 font size setting ("default" case): due to upgrade to 0.90.0
      2009-01-14 fixes for new matplotlib version (0.98.5.2)
'''
import matplotlib
try:
   matplotlib.use('Agg')         # == no "live plot", recommendation: Agg
except:
   print "Could not set Agg for backend..."
   pass
from pylab import *
from matplotlib.font_manager import FontProperties


class cPlotBase:
   '''
      Common base (common functions) for plotting classes
   '''
   LISTTYPE=type([])
   TUPLETYPE=type(())

   def __init__(self,size="800,600",dpi=100):
      '''
      '''
      self.size=size
      self.dpi=dpi
      self.fignum=None

   def SetSize(self,size):
      '''
      '''
      self.size=size

   def FormatEq(self,eq,precision=3):
      '''
      '''
      if len(eq) not in [2,3]: return ''
      s0=['%+.*e'%(precision,v) for v in eq]
      for n in range(len(eq)):
         s0[n]=s0[n][0]+'\/'+s0[n][1:]         # add space before the sign
         if n==0:
            if s0[n][0]=='+': s0[n]=s0[n][1:]    # remove the first +
      s1=[]
      for s in s0:
         i=s.find('e')
         if i>0:
            s1.append(r"%s^{%s}\/"%(s[:i+1],s[i+1:]))
         else:
            s1.append(s)      # should not happen
      eq=[]
      eq.append(r"$y\/=%sx"%s1[0])
      if len(s1)==3: eq.append('^2')
      eq.append(r"\/%s\/"%s1[1])
      if len(s1)==3:
         eq.append(r"x\/%s"%s1[2])
      return r"%s$"%''.join(eq)

   def FormatText(self,txt,space='\\hspace{0.3}'):
      '''
         Tries to format the text to mathtext to be called with eval

         ADD: check whether allready is mathtext -> no formatting

         2006-07-26: two spaces \s\s -> hspace!
         2007-11-29  change for matplotlib 0.91, ALSO escape %,[,]
      '''
      ret=txt
      if not len(ret): return ret
#      if ret[0] not in ['"',"'"]: ret="'%s'"%ret
      s=ret.replace('  ',space)
      s=s.replace('','\\ddot{a}')
      s=s.replace('','\\ddot{o}')
      s=s.replace('','\\ddot{A}')
      s=s.replace('','\\ddot{O}')
      s=s.replace('%','\\%')
      s=s.replace('&','')        # TEMPORARY
      s=s.replace('[','(')       # TEMPORARY
      s=s.replace(']',')')       # TEMPORARY
      ret=r"r'$\mathrm{%s}$'"%s
      return ret


   def _fig(self,nplots=1):
      '''
         Return handle to subplot(arc)
         arc: #axes, #rows, #cols
      '''
      w,h=map(float,self.size.split(','))
      f=figure(num=None,figsize=(w/self.dpi,h/self.dpi),dpi=self.dpi)
      self.fignum=f.number
      clf()
      if nplots<10:
         return subplot(nplots*100+11)
      else:
         return subplot(nplots,1,1)

   def _xylabel(self,data,nsubfig,labname):
      '''
         return 'evalstr' for x/y/z labels
      '''
      evalstr=''
      if data.has_key(labname) and (nsubfig==None or type(data[labname])==self.LISTTYPE):
         try:
            if nsubfig!=None:
               lab=data[labname][nsubfig]
            else:
               lab=data[labname]
            if type(lab)==self.LISTTYPE: lab=lab[0]
            if type(lab)==self.TUPLETYPE:    # tuple
               s=self.FormatText(lab[0])
               sopts=','+','.join(lab[1:])
            else:
               s=self.FormatText(lab)
               sopts=''
            if sopts.find('fontsize')<0:
               sopts+=",fontsize=10"
            if labname=='zlabel':
               evalstr='set_ylabel(%s%s)'%(s,sopts)
            else:
               evalstr='%s(%s%s)'%(labname,s,sopts)
         except:
            raise
      return evalstr


   def _axhline(self,data,nsubfig=None):
      '''
         Draw horizontal line at y==0 (return "evalstr"),
         if data has key 'axhline'

         Created 2006-09-25

         Revision history
            2006-11-15  color="gray" added
      '''
      evalstr=''
      if data.has_key('axhline') and (nsubfig==None or type(data['axhline'])==self.LISTTYPE):
         try:
            if nsubfig!=None:
               lab=data['axhline'][nsubfig]
            else:
               lab=data['axhline']
            evalstr='axhline(%s,color="gray")'%(lab,)
         except:
            raise
      return evalstr



   def _title(self,data,nsubfig=None,ncols=1):
      '''
         return 'evalstr' for title
      '''
      evalstr=None
      try:
         tle=data['title']
         if type(tle)==self.LISTTYPE or nsubfig==None or nsubfig==0:
            fig=True
            if type(tle)==self.LISTTYPE:
               tle=tle[nsubfig]
               fig=False
            if type(tle)==self.TUPLETYPE:
               s=self.FormatText(tle[0])
               sopts=','+','.join(tle[1:])
            else:
               s=self.FormatText(tle)
               sopts=''
            if sopts.find('fontsize')<0:
               sopts+=",fontsize=10"
            if not fig or ncols==1:
               evalstr='%s(%s%s)'%('title',s,sopts)
               eval(evalstr)
            else:
               sopts+=",horizontalalignment='center'"
               evalstr="gcf().text(.5,.92,%s%s)"%(s,sopts)
      except:
         pass
      return evalstr


   def _setTickLabelSize(self,ax,data,nsubfig,labname):
      '''
         Set x/y tick label size
         Return None, if none was set
      '''
      if data.has_key(labname):
         try:
            if type(data[labname])==self.LISTTYPE:
               try:
                  ts=data[labname][nsubfig]
                  if labname[0]=='y':
                     for lbl in ax.get_yticklabels(): lbl.set_size(ts)
                  else:
                     for lbl in ax.get_xticklabels(): lbl.set_size(ts)
                  return True
               except:
                  pass
            else:
               if labname[0]=='y':
                  for lbl in ax.get_yticklabels(): lbl.set_size(data[labname])
               else:
                  for lbl in ax.get_xticklabels(): lbl.set_size(data[labname])
               return True
         except:
            pass
      return None

   def setylims(self,ax,maxydiff,y):
      '''
         set ylims so that values outside maxydiff[0] percentile +- maxydiff[1]
         are not shown (in most cases)

         Revision history:
            2006-09-25  Return (set) ylims
      '''
      ylim=list(ax.get_ylim())
      if not maxydiff or not type(maxydiff)==self.TUPLETYPE or len(maxydiff)!=2 or len(y)<3:
         return ylim
      adj=False
      ys=sort(y)
      ind=int(len(ys)*maxydiff[0]/100.)
      ymx=ys[-ind] ; ymn=ys[ind]
      ymnind=0 ; ymxind=-1
#      print maxydiff,ylim,ymn,ymx   #debug
      if ymn-ylim[0]>maxydiff[1]:   # adjust ymin
         adj=True
         while ymn-ys[ymnind]>maxydiff[1]: ymnind+=1
      if ylim[1]-ymx>maxydiff[1]:   # adjust ymax
         adj=True
         while ys[ymxind]-ymx>maxydiff[1]: ymxind-=1

#      print adj,ymnind,ymxind
      if not adj: return ylim
      vmin=ys[ymnind] ; vmax=ys[ymxind]
      expo,rem=divmod(log10(vmax-vmin),1)
#      print adj,ymnind,ymxind,vmin,vmax,expo,rem
      if rem< 0.5: expo-=1
      scale=10**(-expo)
      vmin=floor(scale*vmin)/scale
      vmax=ceil(scale*vmax)/scale
#      print scale,expo,vmin,vmax
      ax.set_ylim((vmin,vmax))
      return [vmin,vmax]


   def _data(self,dadi):
      '''
         return array(x),array(y), where y[0] is the first data
         vector. Can handle dadi['y'] whether data is in row first
         or column first format.
      '''
      xarr=array(dadi['x'])
      lenx=len(xarr)
      y=array(dadi['y'])
      yshape=shape(y)
      if lenx not in yshape:
         raise Exception,"x length (%d) != y length (%s)"%(lenx,str(yshape))
      if lenx==yshape[0]:  # must transpose
         y=transpose(y)
      return xarr,y


######################################################################

class cPlotXY(cPlotBase):
   '''
      Plot x-y graphs
   '''

   def __init__(self,size="800,600",dpi=100):
      '''
      '''
      cPlotBase.__init__(self,size=size,dpi=dpi)


   def PlotXnY(self,data,figfile):
      '''
         Plot multiple curves to the same fig

         data is a dictionary:
            'x':  common x coordinates as a list OR [[]]
            'y': y coordinates as a list or lists
            'legend': list of legend labels
            'xlabel':   x axis label
            'ylabel':   y axis label
            'title':    title
            'grid': contains 'x','y' or 'xy'
            'xticklabelsize'
            'yticklabelsize'
            'legendfontsize'
            'legendloc' 0 best, 1 upper right (default), ...
            'yminmax': [ymin,ymax]
            'xminmax': [xmin,xmax]
            'kwargs': [{}] to be passed to plot command (list of dictionaries)
               linestyle (ls):  '-','--','-.',':','steps','None'
               linewidth (lw)
               marker : '+' |',' | '.' | '1' | '2' | '3' | '4' | '<' | '>' | 'D' | 'H' |
                        '^' | '_' | 'd' | 'h' | 'o' | 'p' | 's' | 'v' | 'x' | '|'
               markeredgecolor (mec)
               markeredgewidth (mew)
               markerfacecolor (mfc)
               markersize      (ms)
               color (c): line color
            'frame': {"left":,"bottom":,"right":,"wspace":,"hspace":}
            'axhline': []
            'text': (x,y,s,kwargs)
            'line': (a,b) IF singleplot

         Initial version 2006-09-21/22

         Revision history:
            2006-10-09  +axhline
            2006-11-14  +'text'
            2008-05-28  also multidim x possible to use
            2009-02-06  simple x,y plot, 'line'
            2009-06-08  plot also if y shorter than x (using x vals from beginning)
            2009-06-15  fix bug made by previous change
      '''
      ax=self._fig()
      lns=[]
      dax=data['x']
      datay=data['y']
      singleplot=False
      try:
         shape(dax)
         shape(datay)
         ok=True
      except:
         ok=False

      if ok and shape(dax)==shape(datay) and len(shape(dax))==1:     # simple x,y plot
         singleplot=True
         try:
            kwa=data['kwargs']
            lns.append(plot(dax,datay,**kwa))
         except:
            raise
            lns.append(plot(dax,datay))

      else:
         try:
            dax[0][0]
         except:
            dax=[dax for i in range(len(datay))]

         for ind,da in enumerate(zip(dax,datay)):
            x=array(da[0])
            y=array(da[1])
            try:
               if len(y)<len(x):
                  x=x[:len(y)]
               elif len(x)<len(y):
                  y=y[:len(x)]
            except:
               print "x:",x
               print "y:",y
               print
               raise
            try:
               kwa=data['kwargs'][ind]
            except:
               try:
                  kwa=data['kwargs']
               except:
                  kwa=None
            try:
               if kwa==None:
                  lns.append(plot(x,y))
               else:
                  lns.append(plot(x,y,**kwa))
            except:
               print "jahplotlib.PlotXnY, len(x):%s,len(y):%s"%(len(x),len(y))
               raise

      a=gca()

      try:
         li=data['line']
         if singleplot:
            x1=min(dax) ; x2=max(dax)
            y1=x1*li[1]+li[0] ; y2=x2*li[1]+li[0]
            plot([x1,x2],[y1,y2],'k-')
      except:
         pass

      try:
         ymm=data['yminmax']
         a.set_ylim(ymm[0],ymm[1])
      except:
         pass
      try:
         xmm=data['xminmax']
         a.set_xlim(xmm[0],xmm[1])
      except:
         pass

      try:
         data['legend']
         try:
            loc=data['legendpos']
         except:
            loc=0
         try:
            prop=FontProperties(size=data['legendfontsize'])
         except:
            prop=None
         leg=legend(tuple(data['legend']),prop=prop,loc=loc)
         leg.get_frame().set_alpha(0.5)
      except:
         pass
      try:
         if 'x' in data['grid']: ax.xaxis.grid(True,color='gray')
         if 'y' in data['grid']: ax.yaxis.grid(True,color='gray')
      except:
         pass
      evalstr=self._title(data)
      if evalstr: eval(evalstr)
      evalstr=self._xylabel(data,None,'xlabel')
      if evalstr: eval(evalstr)
      evalstr=self._xylabel(data,None,'ylabel')
      if evalstr: eval(evalstr)

      if not self._setTickLabelSize(ax,data,None,'yticklabelsize'):
         sz=8   # set tick label font size to smaller
         for lbl in ax.get_yticklabels(): lbl.set_size(sz)
      if not self._setTickLabelSize(ax,data,None,'xticklabelsize'):
         sz=8
         for lbl in ax.get_xticklabels(): lbl.set_size(sz)

      try:
         subplots_adjust(**data['frame'])
      except:
         pass
      evalstr=self._axhline(data)
      if evalstr:
         eval(evalstr)

      try:
         te=list(data['text'])
         try:
            te[0][0]
         except:
            te=[te]
         for t in te:
            t=list(t)
            t[2]=self.FormatText(t[2])
            if len(t)<4: t.append({'va':'center'})
            evalstr="gcf().text(%s,%s,%s,%s)"%(t[0],t[1],t[2],t[3])
            eval(evalstr)
      except:
         pass

      try:
         ymm=data['yminmax']
         a.set_ylim(ymm[0],ymm[1])
      except:
         pass
      savefig(figfile,dpi=self.dpi)


   def PlotErrorbar(self,data,figfilenames,onecol=False,onefig=False):
      '''
         Plot subplots with (optional) error bars (if data's 'y' is 2-dim array) with
         common 'x'.

         maxydiff=(percentile,maxdiff)

         data is a dictionary:
            'x': []     x-data as a list, OR list of lists
            'y': [[]]   y-data as a list (of lists): [[0]=1st data col]
            'yerr': [[]] symmetric error bars
            'kwargs': [{}] to be passed to plot command (list of dictionaries)
               linestyle (ls):  '-','--','-.',':','steps','None'
               linewidth (lw)
               marker : '+' |',' | '.' | '1' | '2' | '3' | '4' | '<' | '>' | 'D' | 'H' |
                        '^' | '_' | 'd' | 'h' | 'o' | 'p' | 's' | 'v' | 'x' | '|'
               markeredgecolor (mec)
               markeredgewidth (mew)
               markerfacecolor (mfc)
               markersize      (ms)
               color (c): line color
            'grid': contains 'x','y' or 'xy' effects ALL subplots
            'xlabel': written under the bottom most subfigs
            'title': [()] list of titles for subplots, OR only one for the whole fig
            'ylabel': [()] list of y-axis labels (mathtext), with optional args ('txt',arg)
            'yformatter': [] list of parameters for FormatStrFormatter
            'yminmax':  []
            'xminmax': []
            'yticks': []
            'ylog': []
            'xlabel': []
            'yminminmaxmax: [(min,max)]   min min and max max limits for y-scale
            'yadjlims': [] [(percentile,maxdiff)]
            'yticklabelsize': []
            'xticklabelsize': []
            'frame': {"left":,"bottom":,"right":,"wspace":,"hspace":}
            'axhline': []
            'lineat':[] # draw a line at yval lineat
            'xdate':True

         Revision history:
            2006-10-09  copy/adjust from PlotMultiDtY
            2007-10-05  + 'frame'
            2007-10-10  multi x, 'xdate', 'lineat'
            2008-02-22  'xminmax'
            2008-08-27  update: +'onefig'
      '''
      if data==None: return
      ny=len(data['y'])
      if ny<=0: return
      dax=data['x']
      try:
         dax[0][0]
      except (IndexError,TypeError):
         dax=[]
         for i in range(ny): dax.append(data['x'])

      ncols=1
      if not onefig and not onecol and ny>=4: ncols=2
      nrows=int(round(ny/float(ncols)))

      xgrid=False ; ygrid=False
      if 'grid' in data.keys():  # set grid to gray - must be a simpler way to do this
         if 'x' in data['grid']: xgrid=True
         if 'y' in data['grid']: ygrid=True
      nyy=ny
      if onefig: nyy=1
      ax=self._fig(nyy)

      if data.has_key('xdate') and data['xdate']:
         minx=9e99; maxx=0.3
         for x in dax:
            if min(x)<minx: minx=min(x)
            if max(x)>maxx: maxx=max(x)
         dxd=maxx-minx
         xmm=[int(minx)-1, int(maxx)+1]
         if dxd<2:
            majLocator=HourLocator(interval=4)  # interval=2 -> 4, 2007-06-14
            minLocator=None
            minLocator=HourLocator(interval=1)
            dateFormatter=DateFormatter("%H:%M")
         elif dxd<15:
            majLocator=DayLocator()
            minLocator=HourLocator(12)
            dateFormatter = DateFormatter('%m-%d')
         elif dxd<32:
            majLocator=DayLocator(interval=5)
            minLocator=DayLocator()
            dateFormatter = DateFormatter('%m-%d')
         elif dxd<182:
            majLocator=MonthLocator()
            minLocator=DayLocator(interval=5)
            dateFormatter = DateFormatter('%m-%d')
         elif dxd<366: # added 2007-08-16
            majLocator=MonthLocator(interval=2)
            minLocator=MonthLocator()
            dateFormatter = DateFormatter("%Y-%m-%d")
         elif dxd<1000: # added 2007-08-24
            majLocator=MonthLocator(interval=3)
            minLocator=MonthLocator()
            dateFormatter = DateFormatter("%Y-%m-%d")
         else:
            majLocator=YearLocator()
            minLocator=None
            dateFormatter=DateFormatter("%Y")
      else:
         xmm=None
         majLocator=None
         minLocator=None
         dateFormatter=None


      for nsubfig in range(ny):
         if nsubfig==0 or not onefig:
            row=nsubfig/(ny/nrows)+1
        #         ax=subplot(nrows,ncols,nsubfig+1)
            nr=nrows
            if onefig:nr=1
            ax=subplot(100*nr+11+nsubfig)

         ####################################
         try:
            kwa=data['kwargs'][nsubfig]
         except:
            kwa={'mec':"blue",'ms':2}
         x=dax[nsubfig]
         y1=list(data['y'][nsubfig])      # for 'None' handling (!??)
         if data.has_key('yerr'):
            ye=data['yerr'][nsubfig]
            errorbar(x,y1,ye,**kwa)
         else:
            plot(x,y1,**kwa)
         try:
            ymm=[data['lineat'][nsubfig],data['lineat'][nsubfig]]
            plot(xmm,ymm,**{'ls':'-','color':'darkred','lw':1.5})
         except:
            pass
         ####################################

         if xgrid: ax.xaxis.grid(True,color='gray')
         if ygrid: ax.yaxis.grid(True,color='gray')

         if majLocator: ax.xaxis.set_major_locator(majLocator)
         if minLocator: ax.xaxis.set_minor_locator(minLocator)

         if row==nrows or (onefig and nsubfig==ny-1):
            if dateFormatter: ax.xaxis.set_major_formatter(dateFormatter)
            evalstr=self._xylabel(data,nsubfig,'xlabel')
            if evalstr: eval(evalstr)
         else:
            ax.xaxis.set_major_formatter(NullFormatter())
         ax.yaxis.set_major_locator(MaxNLocator(nbins=5))

         try:
            ax.yaxis.set_major_formatter(FormatStrFormatter(data['yformatter'][nsubfig]))
         except:
            ax.yaxis.set_major_formatter(ScalarFormatter(useOffset=False)) #,useMathText=True))

         evalstr=self._xylabel(data,nsubfig,'ylabel')
         if evalstr: eval(evalstr)
         evalstr=self._title(data,nsubfig,ncols)
         if evalstr: eval(evalstr)

         a=gca()

         if not self._setTickLabelSize(ax,data,nsubfig,'yticklabelsize'):
            for lbl in ax.get_yticklabels(): lbl.set_size(8)
         if not self._setTickLabelSize(ax,data,nsubfig,'xticklabelsize'):
            for lbl in ax.get_xticklabels(): lbl.set_size(8)

         try:
            ymm=data['yminmax'][nsubfig]
            a.set_ylim(ymm[0],ymm[1])
            ylims=ymm
         except:
            ylims=list(a.get_ylim())
            try:
               ymm=data['yminminmaxmax'][nsubfig]
               if ymm[0]:
                  if ylims[0]<ymm[0]: ylims[0]=ymm[0]
               if ymm[1]:
                  if ylims[1]>ymm[1]: ylims[1]=ymm[1]
               a.set_ylim(ylims[0],ylims[1])
            except:
               try:
                  ylims=self.setylims(a,data['yadjlims'][nsubfig],y1)
               except:
                  pass

         try:
            xmm=data['xminmax']
            a.set_xlim(xmm[0],xmm[1])
         except:
            pass

         evalstr=self._axhline(data,nsubfig)
         if evalstr:
            if ylims[0]<0 and ylims[1]>0:
               eval(evalstr)
               # from some reason axhline resets ylims: set again...
               a.set_ylim(ylims[0],ylims[1])

         try:
            yt=data['yticks'][nsubfig]
            if yt:
               try: yticks(yt[0],yt[1])
               except: yticks(yt)
         except:
            pass

         try:
            yt=data['ylog'][nsubfig]
            if yt: ax.set_yscale('log')
         except:
            pass

         if dateFormatter: # x lims
            a.set_xlim(min(int(minx-.01*dxd),int(minx)-1),max(int(maxx+.01*dxd),int(maxx)+2))


      if ncols>1: subplots_adjust(wspace=.3)
      try:
         subplots_adjust(**data['frame'])
      except:
         pass

      try:
         for figfile in figfilenames:
            savefig(figfile,dpi=self.dpi)
      except:
         savefig(figfilenames,dpi=self.dpi)




##################################################################


class cPlotDtY(cPlotBase):
   '''
      Plot datetime-y graphs

      Revision history:
   '''

   def __init__(self,size="800,600",dpi=100):
      '''
      '''
      cPlotBase.__init__(self,size=size,dpi=dpi)


   def PlotDtContour(self,figfilenames,data,yscale_log=True,miny=None,maxy=500):
      '''
      Revision history:
         2007-11-27 +miny,maxy, ['frame']
         2007-11-29 'zfilter':(minvals,maxvals), default: (0,0), 'zminmax':(minv,maxv)
      '''
      sb=self._fig()
      majLocator=DayLocator()
      minLocator=HourLocator(12)
      dateFormatter = DateFormatter('%d.%m')

      try:
         zminset,zmaxset=data['zminmax']
      except:
         zminset,zmaxset=None,None
      try:
         minfil,maxfil=data['zfilter']
      except:
         minfil,maxfil=0,0

      zdata=[]
      if zminset!=None or zmaxset!=None or minfil>0 or maxfil>0:   # must adjust z data
         if zminset!=None: minfil=0
         if zmaxset!=None: maxfil=0
         zmins=[1e33 for i in range(minfil)]
         zmaxs=[0. for i in range(maxfil)]
         for zd in data['z']:
            mn=min(zd) ; mx=max(zd) ; za=None
            if (mn<zminset and zminset!=None) or (zmaxset!=None and mx>zmaxset):
               if zminset!=None and mn<zminset:
                  za=[max(z,zminset) for z in zd]
               else:
                  za=[z for z in zd]
               if zmaxset!=None and mx>zmaxset:
                  za=[min(z,zmaxset) for z in za]
            else:
               za=[z for z in zd]
               if minfil>0 or maxfil>0:
                  for z in zd:
                     if minfil>0 and z<zmins[-1]:
                        zmins[-1]=z
                        zmins.sort()
                     if maxfil>0 and z>zmaxs[0]:
                        zmaxs[0]=z
                        zmaxs.sort()
            zdata.append(za)
         if minfil>0 or maxfil>0:
            zminset=None ; zmaxset=None
            if minfil>0:
               try:
                  if minfil==1 or (minfil>1 and (zmins[-1]-zmins[0])/abs(zmins[-1])>.05):
                     zminset=zmins[-1]
               except:
                  pass
            if maxfil>0:
               try:
                  if maxfil==1 or (maxfil>1 and (zmaxs[-1]-zmaxs[0])/abs(zmaxs[0])>.05):
                     zmaxset=zmaxs[0]
               except:
                  pass

         if zminset>None or zmaxset>None:
            zdatan=[]
            for za in zdata:
               if zminset>0 and min(za)<zminset:
                  zan=[max(z,zminset) for z in za]
                  za=zan
               if zmaxset>0 and max(za)>zmaxset:
                  zan=[min(z,zmaxset) for z in za]
                  za=zan
               zdatan.append(za)
            zdata=zdatan
      else:
         zdata=data['z']
      zmin=1e9 ; zmax=0
      for za in zdata:
         if min(za)<zmin: zmin=min(za)
         if max(za)>zmax: zmax=max(za)


#      pc=contourf(array(data['x']),array(data['y']),array(data['z']),shading='flat',cmap=cm.summer)
      pc=pcolor(array(data['x']),array(data['y']),array(zdata),shading='flat',cmap=cm.jet)
      cbar=colorbar()

#      cla()
      ax=gca()
      if yscale_log:
         ax.set_yscale('log')
         if miny and maxy:
            ax.set_ylim(miny,maxy)
         elif maxy:
            ax.set_ylim(int(min(data['y'][0])),maxy)
         elif miny:
            ax.set_ylim(miny,int(max(data['y'][0])))
      try:
         ax.set_ylim(data['yminmax'][0],data['yminmax'][1])
      except:
         pass

      ax.yaxis.set_major_formatter(FormatStrFormatter('%.0f'))

      ax.xaxis_date()
      ax.xaxis.set_major_locator(majLocator)
      ax.xaxis.set_minor_locator(minLocator)
      ax.xaxis.set_major_formatter(dateFormatter)

      xgrid=False ; ygrid=False
      if 'grid' in data.keys():  # set grid to gray - must be a simpler way to do this
         if 'x' in data['grid']: xgrid=True
         if 'y' in data['grid']: ygrid=True
      if xgrid: ax.xaxis.grid(True,color='black')
      if ygrid: ax.yaxis.grid(True,color='black')

      evalstr=self._xylabel(data,None,'xlabel')
      if evalstr: eval(evalstr)
      evalstr=self._xylabel(data,None,'ylabel')
      if evalstr: eval(evalstr)
      evalstr=self._title(data)
      if evalstr: eval(evalstr)
#      evalstr=self._xylabel(data,None,'zlabel')   # results doesn't seem to work?
      if data.has_key('zlabel'):
         cbar.ax.set_ylabel(data['zlabel'],fontsize=10)

      if not self._setTickLabelSize(ax,data,0,'yticklabelsize'):
         sz=8   # set tick label font size to smaller
         for lbl in ax.get_yticklabels(): lbl.set_size(sz)
      if not self._setTickLabelSize(ax,data,0,'xticklabelsize'):
         sz=8
         for lbl in ax.get_xticklabels(): lbl.set_size(sz)

      try:
         subplots_adjust(**data['frame'])
      except:
         pass

      try:
         for figfile in figfilenames:
            savefig(figfile,dpi=self.dpi)
      except:
         savefig(figfilenames,dpi=self.dpi)


   def PlotMultiDtY(self,figfilenames,data=None,onecol=False,transposey=False):
      '''
         Plot subplots (if data's 'y' is 2-dim array) with
         common 'x'.

         maxydiff=(percentile,maxdiff)

         data is a dictionary:
            'x': []     x-data as a list
            'y': [[]]   y-data as a list (of lists)
            'kwargs': [{}] to be passed to plot command (list of dictionaries)
               linestyle (ls):  '-','--','-.',':','steps','None'
               linewidth (lw)
               marker : '+' |',' | '.' | '1' | '2' | '3' | '4' | '<' | '>' | 'D' | 'H' |
                        '^' | '_' | 'd' | 'h' | 'o' | 'p' | 's' | 'v' | 'x' | '|'
               markeredgecolor (mec)
               markeredgewidth (mew)
               markerfacecolor (mfc)
               markersize      (ms)
               color (c): line color
            'grid': contains 'x','y' or 'xy' effects ALL subplots
            'xlabel': written under the bottom most subfigs
            'title': [()] list of titles for subplots, OR only one for the whole fig
            'ylabel': [()] list of y-axis labels (mathtext), with optional args ('txt',arg)
            'yformatter': [] list of parameters for FormatStrFormatter
            'yminmax':  [], 'xminmax': []
            'yticks': []
            'ylog': []
            'xlabel': []
            'yminminmaxmax: [(min,max)]   min min and max max limits for y-scale
            'yadjlims': [] [(percentile,maxdiff)]
            'yticklabelsize': []
            'xticklabelsize': []
            'axhline': []
            'frame': {"left":,"bottom":,"right":,"wspace":,"hspace":}
            'lineat':[[]] # draw a line at yval lineat
            'text': [(x,y,s,kwargs)]


         Revision history:
            2006-08-22  onecol,'yadjlims' added
            2006-09-25  axhline
            2006-10-09  par: transposey
            2006-10-12  'frame'
            2006-11-15  yy!=[] check
            2007-01-08 'xminmax'
            2007-01-10  xlabel fix
            2007-01-12  xscale fix (data vs 'xminmax'), x formatting enhancements
            2007-08-16  xscale formatting with dx>180d added
            2007-08-20  dxd>0 check
            2008-03-05  +'lineat'
            2008-05-08 'text'
      '''
      if data==None: return
      x=data['x']
      if not x or not data['y']: return
      y=array(data['y'])
      if transposey: y=transpose(y)
      try:
         dxd=data['xminmax'][1]-data['xminmax'][0]
      except:
         dxd=x[-1]-x[0]
      try:
         ny=shape(y)[1]
      except:
         ny=1
      assert 0<ny<10
      if dxd<=0: return

      ncols=1
      if not onecol and ny>=4: ncols=2
      nrows=int(round(ny/float(ncols)))
      majLocator=None

      xgrid=False ; ygrid=False
      if 'grid' in data.keys():  # set grid to gray - must be a simpler way to do this
         if 'x' in data['grid']: xgrid=True
         if 'y' in data['grid']: ygrid=True

      ax=self._fig(ny)
      for nsubfig in range(ny):
         row=nsubfig/(ny/nrows)+1
         subplotno=nrows*100+ncols*10+nsubfig+1
         ax=subplot(subplotno)

         if not majLocator:
            if dxd<=1:
               majLocator=HourLocator(interval=4)  # interval=2 -> 4, 2007-06-14
               minLocator=None
               minLocator=HourLocator(interval=1)
               dateFormatter=DateFormatter("%H:%M")
            elif dxd<3:
               majLocator=HourLocator(interval=8)
               minLocator=None
               minLocator=HourLocator(interval=2)
               dateFormatter=DateFormatter("%H:%M")
            elif dxd<15:
               majLocator=DayLocator()
               minLocator=HourLocator(12)
               dateFormatter = DateFormatter('%m-%d')
            elif dxd<32:
               majLocator=DayLocator(interval=5)
               minLocator=DayLocator()
               dateFormatter = DateFormatter('%m-%d')
            elif dxd<182:
               majLocator=MonthLocator()
               minLocator=DayLocator(interval=5)
               dateFormatter = DateFormatter('%m-%d')
            elif dxd<366: # added 2007-08-16
               majLocator=MonthLocator(interval=2)
               minLocator=MonthLocator()
               dateFormatter = DateFormatter("%Y-%m-%d")
            elif dxd<1000: # added 2007-08-24
               majLocator=MonthLocator(interval=3)
               minLocator=MonthLocator()
               dateFormatter = DateFormatter("%Y-%m-%d")
            else: #2014-10-08
               majLocator=YearLocator()   
               minLocator=MonthLocator(interval=6)
               dateFormatter=DateFormatter("%Y")

         ####################################
         try:
            kwa=data['kwargs'][nsubfig]
         except:
            kwa={'mec':"blue",'ms':2}
         y1=list(y[:,nsubfig])      # for 'None' handling (!??)
         # possible None in the data - MUST GET RID of those data points
         # masked array doesn't seem to work (numpy.ma.masked_where)...
         xx=[] ; yy=[]
         for i1,i2 in zip(x,y1):
            if i2==None: continue
            xx.append(i1); yy.append(i2)
         if yy!=[]:
#            print kwa
            try:
               plot_date(xx,yy,**kwa)
            except:
               print "\nEXCEPTION: jahplotlib problems (matplotlib plot_date()): check that parameters are numbers!\n"
               raise

         ####################################

         if xgrid: ax.xaxis.grid(True,color='gray')
         if ygrid: ax.yaxis.grid(True,color='gray')
         ax.xaxis.set_major_locator(majLocator)
         if minLocator: ax.xaxis.set_minor_locator(minLocator)
#         print nsubfig,row,subplotno
         if row==nrows:
            ax.xaxis.set_major_formatter(dateFormatter)
            evalstr=self._xylabel(data,None,'xlabel')
            if evalstr:
               eval(evalstr)
#               print evalstr  #debug
         else:
            ax.xaxis.set_major_formatter(NullFormatter())
         ax.yaxis.set_major_locator(MaxNLocator(nbins=5))

         try:
            ax.yaxis.set_major_formatter(FormatStrFormatter(data['yformatter'][nsubfig]))
         except:
            ax.yaxis.set_major_formatter(ScalarFormatter(useOffset=False)) #,useMathText=True))

         evalstr=self._xylabel(data,nsubfig,'ylabel')
         if evalstr:
            eval(evalstr)
#            print "evalstr:'%s'"%(evalstr,)  #debug
         evalstr=self._title(data,nsubfig,ncols)
         if evalstr:
            eval(evalstr)
#            if matplotlib.__version__[:4]=='0.98':
#               ax.title.set_y(0.91)
#            print "evalstr:'%s'"%(evalstr,)  #debug

         try:
            yt=data['ylog'][nsubfig]
            if yt: ax.set_yscale('log')
         except:
            pass

         a=gca()
         maxx=max(x)
         if dxd>=182:
            a.set_xlim(int(min(x)-5),int(max(x)+5))
         elif dxd>=1:
            if 0<maxx%1<=.5: maxx=int(maxx)+.5
            elif maxx%1>.5: maxx=int(maxx)+1
            a.set_xlim(int(min(x)),maxx)
         else:
            pass

         if not self._setTickLabelSize(ax,data,nsubfig,'yticklabelsize'):
            if ncols>1: sz=7
            else: sz=8   # set tick label font size to smaller
            for lbl in ax.get_yticklabels(): lbl.set_size(sz)
         if not self._setTickLabelSize(ax,data,nsubfig,'xticklabelsize'):
            if ncols>1 and row==nrows:
               sz=6
            else:
               sz=8
            for lbl in ax.get_xticklabels(): lbl.set_size(sz)

         try:
            ymm=data['yminmax'][nsubfig]
            a.set_ylim(ymm[0],ymm[1])
            ylims=ymm
         except:
            ylims=list(a.get_ylim())
            try:
               ymm=data['yminminmaxmax'][nsubfig]
               if ymm[0]!=None:
                  if ylims[0]<ymm[0]: ylims[0]=ymm[0]
               if ymm[1]!=None:
                  if ylims[1]>ymm[1]: ylims[1]=ymm[1]
               a.set_ylim(ylims[0],ylims[1])
            except:
               try:
                  ylims=self.setylims(a,data['yadjlims'][nsubfig],yy)
               except:
                  pass

         try:
            xmm=data['xminmax']
            a.set_xlim(xmm[0],xmm[1])
         except:
            xmm=a.get_xlim()

         try:
            ymm=[data['lineat'][nsubfig],data['lineat'][nsubfig]]
            xm=list(xmm)
            xm[0]+=.1 ; xm[1]-=.1
            plot(xm,ymm,**{'ls':'-','color':'darkred','lw':1.2})
            a.set_xlim(xmm[0],xmm[1])
         except:
            pass


         evalstr=self._axhline(data,nsubfig)
         if evalstr:
            if ylims[0]<0 and ylims[1]>0:
               eval(evalstr)
               # from some reason axhline resets ylims: set again...
               a.set_ylim(ylims[0],ylims[1])

         try:
            yt=data['yticks'][nsubfig]
            if yt:
               try: yticks(yt[0],yt[1])
               except: yticks(yt)
         except:
            pass

         try:
            te=list(data['text'][nsubfig])
            try:
               te[0][0]
            except:
               te=[te]
            for t in te:
               t=list(t)
               t[2]=self.FormatText(t[2])
               if len(t)<4: t.append({'va':'center','transform':'gca().transAxes'})
               evalstr="gcf().text(%s,%s,%s,%s)"%(t[0],t[1],t[2],t[3])
               eval(evalstr)
         except:
            pass


      try:
         subplots_adjust(**data['frame'])
      except:
         if ncols>1: subplots_adjust(wspace=.3)
      try:
         for figfile in figfilenames:
            savefig(figfile,dpi=self.dpi)
      except:
         savefig(figfilenames,dpi=self.dpi)



   def PlotDtnY(self,data,figfile):
      '''
         Plot multiple curves to the same fig

         data is a dictionary:
            'x':  common x coordinates as a list
            'y': y coordinates as a list or lists
            'legend': list of legend labels
            'xlabel':   x axis label
            'ylabel':   y axis label
            'title':    title
            'grid': contains 'x','y' or 'xy'
            'xticklabelsize'
            'yticklabelsize'
            'legendfontsize'
            'yminmax':  []
            'kwargs': [{}] to be passed to plot command (list of dictionaries)
               linestyle (ls):  '-','--','-.',':','steps','None'
               linewidth (lw)
               marker : '+' |',' | '.' | '1' | '2' | '3' | '4' | '<' | '>' | 'D' | 'H' |
                        '^' | '_' | 'd' | 'h' | 'o' | 'p' | 's' | 'v' | 'x' | '|'
               markeredgecolor (mec)
               markeredgewidth (mew)
               markerfacecolor (mfc)
               markersize      (ms)
               color (c): line color
            'frame': {"left":,"bottom":,"right":,"wspace":,"hspace":}
            'axhline': []
            'lineat':[[]] # draw a line at yval lineat
            'text': [(x,y,s,kwargs)]

         Initial version 2006-10-17

         Revision history:
            2006-10-09  +axhline
            2007-08-20  dxd>0 check
            2009-04-07  +lineat, text
      '''
      x,ydata=self._data(data)
      ax=self._fig()
      lns=[]
      for ind,ylst in enumerate(ydata):
         y=array(ylst)
         try:
            kwa=data['kwargs'][ind]
         except:
            kwa={'mec':"blue",'ms':1.5}
         lns.append(plot_date(x,y,**kwa))

      try:
         data['legend']
         try:
            loc=data['legendpos']
         except:
            loc=0
         if data.has_key('legendfontsize'):
            fp=FontProperties(size=data['legendfontsize'])
            leg=legend(tuple(data['legend']),prop=fp,loc=loc)
         else:
            leg=legend(tuple(data['legend']),loc=loc)
         leg.get_frame().set_alpha(0.5)
      except:
         pass
      try:
         if 'x' in data['grid']: ax.xaxis.grid(True,color='gray')
         if 'y' in data['grid']: ax.yaxis.grid(True,color='gray')
      except:
         pass

      dxd=x[-1]-x[0]

      if dxd<=0: return
      if dxd<1:
         majLocator=HourLocator(interval=2)
         minLocator=None
         minLocator=HourLocator(interval=1)
         dateFormatter=DateFormatter("%H:%M")
      elif dxd<15:
         majLocator=DayLocator()
         minLocator=HourLocator(12)
         dateFormatter = DateFormatter('%m-%d')
      elif dxd<32:
         majLocator=DayLocator(interval=5)
         minLocator=DayLocator()
         dateFormatter = DateFormatter('%m-%d')
      elif dxd<182:
         majLocator=MonthLocator()
         minLocator=DayLocator(interval=5)
         dateFormatter = DateFormatter('%m-%d')
      elif dxd<366: # added 2007-08-16
         majLocator=MonthLocator(interval=2)
         minLocator=MonthLocator()
         dateFormatter = DateFormatter("%Y-%m-%d")
      else: # added 2007-08-24
         majLocator=MonthLocator(interval=3)
         minLocator=MonthLocator()
         dateFormatter = DateFormatter("%Y-%m-%d")

      ax.xaxis.set_major_locator(majLocator)
      if minLocator: ax.xaxis.set_minor_locator(minLocator)
      ax.xaxis.set_major_formatter(dateFormatter)
      a=gca()
      a.xaxis_date()
      maxx=max(x)
      if dxd>=1:
         if 0<maxx%1<=.5: maxx=int(maxx)+.5
         elif maxx%1>.5: maxx=int(maxx)+1
         a.set_xlim(int(min(x)),maxx)
      try:
         a.set_ylim(data['yminmax'][0],data['yminmax'][1])
      except:
         pass

      try:
         ymm=[data['lineat'],data['lineat']]
         xm=list(a.get_xlim())
         xm[0]+=.1 ; xm[1]-=.1
         plot(xm,ymm,**{'ls':'-','color':'darkgray','lw':1.2})
#         a.set_xlim(xmm[0],xmm[1])
      except:
         pass
#         raise
      try:
         te=list(data['text'])
         try:
            te[0][0]
         except:
            te=[te]
         for t in te:
            t=list(t)
            t[2]=self.FormatText(t[2])
            if len(t)<4: t.append({'va':'center'})
            evalstr="gcf().text(%s,%s,%s,%s)"%(t[0],t[1],t[2],t[3])
            eval(evalstr)
      except:
         pass

      evalstr=self._title(data)
      if evalstr: eval(evalstr)
      evalstr=self._xylabel(data,None,'xlabel')
      if evalstr: eval(evalstr)
      evalstr=self._xylabel(data,None,'ylabel')
      if evalstr: eval(evalstr)
      self._setTickLabelSize(ax,data,None,'xticklabelsize')
      self._setTickLabelSize(ax,data,None,'yticklabelsize')
      try:
         subplots_adjust(**data['frame'])
      except:
         pass

      evalstr=self._axhline(data)
      if evalstr:  eval(evalstr)

      savefig(figfile,dpi=self.dpi)



   def PlotnDtnY(self,figfile,data):
      '''
         Plot multiple curves to the same fig

         data is a dictionary:
            'x': [[x],[x],...]
            'y': [[y],[y],...]
            'legend': list of legend labels
            'xlabel':   x axis label
            'ylabel':   y axis label
            'title':    title
            'grid': contains 'x','y' or 'xy'
            'xticklabelsize'
            'yticklabelsize'
            'legendfontsize'
            'yminmax':  []
            'kwargs': [{}] to be passed to plot command (list of dictionaries)
               linestyle (ls):  '-','--','-.',':','steps','None'
               linewidth (lw)
               marker : '+' |',' | '.' | '1' | '2' | '3' | '4' | '<' | '>' | 'D' | 'H' |
                        '^' | '_' | 'd' | 'h' | 'o' | 'p' | 's' | 'v' | 'x' | '|'
               markeredgecolor (mec)
               markeredgewidth (mew)
               markerfacecolor (mfc)
               markersize      (ms)
               color (c): line color
            'frame': {"left":,"bottom":,"right":,"wspace":,"hspace":}
            'axhline': []

         Initial version 2006-10-17

         Revision history:
            2006-10-09  +axhline
            2007-08-20  dxd>0 check

      '''
      assert len(data['x'])==len(data['y'])
      ax=self._fig()
      lns=[] ; ind=0
      for xlst,ylst in zip(data['x'],data['y']):
         x=array(xlst)
         y=array(ylst)
         try:
            kwa=data['kwargs'][ind]
         except:
            kwa={'mec':"blue",'ms':1.5}
         lns.append(plot_date(x,y,**kwa))
         ind+=1

      try:
         data['legend']
         try:
            loc=data['legendpos']
         except:
            loc=0
         if data.has_key('legendfontsize'):
            fp=FontProperties(size=data['legendfontsize'])
            leg=legend(tuple(data['legend']),prop=fp,loc=loc)
         else:
            leg=legend(tuple(data['legend']),loc=loc)
         leg.get_frame().set_alpha(0.5)
      except:
         pass
      try:
         if 'x' in data['grid']: ax.xaxis.grid(True,color='gray')
         if 'y' in data['grid']: ax.yaxis.grid(True,color='gray')
      except:
         pass

      dxd=x[-1]-x[0]
      if dxd<=0: return
      if dxd<1:
         majLocator=HourLocator(interval=2)
         minLocator=None
         minLocator=HourLocator(interval=1)
         dateFormatter=DateFormatter("%H:%M")
      elif dxd<15:
         majLocator=DayLocator()
         minLocator=HourLocator(12)
         dateFormatter = DateFormatter('%m-%d')
      elif dxd<32:
         majLocator=DayLocator(interval=5)
         minLocator=DayLocator()
         dateFormatter = DateFormatter('%m-%d')
      else:
         majLocator=MonthLocator()
         minLocator=DayLocator(interval=5)
         dateFormatter = DateFormatter('%m-%d')
      ax.xaxis.set_major_locator(majLocator)
      if minLocator: ax.xaxis.set_minor_locator(minLocator)
      ax.xaxis.set_major_formatter(dateFormatter)
      a=gca()
      a.xaxis_date()

      maxx=max(x)
      if dxd>=182:
         a.set_xlim(int(min(x)-5),int(max(x)+5))
      elif dxd>=1:
         if 0<maxx%1<=.5: maxx=int(maxx)+.5
         elif maxx%1>.5: maxx=int(maxx)+1
         a.set_xlim(int(min(x)),maxx)
      else:
         pass
      try:
         xmm=data['xminmax']
         a.set_xlim(xmm[0],xmm[1])
      except:
         pass
      try:
         a.set_ylim(data['yminmax'][0],data['yminmax'][1])
      except:
         pass


      evalstr=self._title(data)
      if evalstr: eval(evalstr)
      evalstr=self._xylabel(data,None,'xlabel')
      if evalstr: eval(evalstr)
      evalstr=self._xylabel(data,None,'ylabel')
      if evalstr: eval(evalstr)
      self._setTickLabelSize(ax,data,None,'xticklabelsize')
      self._setTickLabelSize(ax,data,None,'yticklabelsize')
      try:
         subplots_adjust(**data['frame'])
      except:
         pass

      evalstr=self._axhline(data)
      if evalstr:  eval(evalstr)

      savefig(figfile,dpi=self.dpi)

