'''
   jahradaktlib.py
'''
import sys,math
import mx.DateTime as DateTime

DEFBGCNTS=280     # was 240 until 2005-11-23

class cRnConc:
   """
   Methods for calculating Rn concentration from
   measured counts and flow for ONE (and two?)
   filter beta counting system used at FMI

   Initial version 2002-08-12 byjah
   Revision history
      2005-02-17  minor fixes
   """
   # Decay constant (lambda) in 1/s !!! (3.05, 26.8, 19.8 min)
   DECAY_CONST=[3.78769e-3, 0.431062e-3,0.583457e-3]

   # Default Pb-214, Bi-214 beta counting efficiencies
   BetaEff=[.0096,.043]      # always used
#   BetaEff=[1.7*.0096,1.7*.043]    # 1.7*

   QTERMS={
      'Q12':DECAY_CONST[0]/(DECAY_CONST[0]-DECAY_CONST[1]),
      'Q13':DECAY_CONST[0]/(DECAY_CONST[0]-DECAY_CONST[2]),
      'Q21':DECAY_CONST[1]/(DECAY_CONST[1]-DECAY_CONST[0]),
      'Q23':DECAY_CONST[1]/(DECAY_CONST[1]-DECAY_CONST[2]),
      'Q31':DECAY_CONST[2]/(DECAY_CONST[2]-DECAY_CONST[0]),
      'Q32':DECAY_CONST[2]/(DECAY_CONST[2]-DECAY_CONST[1])
   }

   STerms={}

   def __init__(self, betaeff=BetaEff):
      self.beff=betaeff
      self.aonf=[0.,0.,0.]
      self.ptins=0
      self.dtp=DateTime.localtime(0)
      self.Cp=0
      self.STerms={}


   def GetStateFromDb(self, db, tblname="_rn_calc"):
      """
      Get the state from db. Return last dt
      """
      dbcu=db.cursor()
      s="SELECT * FROM %s;" % tblname
      try:
         dbcu.execute(s)
      except:
         print "GetStateFromDb - no luck!"
         dbcu.close()
         return self.dtp
      da=dbcu.fetchall()
      dbcu.close()
      if da:
         self.dtp=da[0][0]
         self.Cp=da[0][1]
         self.aonf[0]=da[0][2]
         self.aonf[1]=da[0][3]
         self.aonf[2]=da[0][4]
      else:
         self.__init__()
         return None
      return self.dtp

   def SetStateToDb(self, db, tblname="_rn_calc"):
      """
      Sets the state (last calc. value) to db
      """
      dbcu=db.cursor()
      s="""
         INSERT INTO %s VALUES ('%s',%.2f,%.2f,%.2f,%.2f,'%s');
      """ % (tblname, self.dtp, self.Cp, self.aonf[0], self.aonf[1],
         self.aonf[2], DateTime.now())
      try:
         dbcu.execute("DELETE FROM %s;" % tblname)
      except:
         pass
      dbcu.execute(s)
      dbcu.close()
      db.commit()


   def DecayCounts(self, tins=600):
      """
      Returns counts detected within time tins from the activity
      already on filter.

      2002-12-03 BUG FIX: c3: exp(-self.DECAY_CONST): time (tins) WAS MISSING!

      """
      c2=self.aonf[1]/self.DECAY_CONST[1]* \
         (1-math.exp(-self.DECAY_CONST[1]*tins))+ \
         self.QTERMS['Q21']*self.aonf[0]* \
         ((1-math.exp(-self.DECAY_CONST[0]*tins))/ self.DECAY_CONST[0]- \
         (1-math.exp(-self.DECAY_CONST[1]*tins))/ \
         self.DECAY_CONST[1])
      c2*=self.beff[0]
      c3=self.aonf[2]/self.DECAY_CONST[2]* \
         (1-math.exp(-self.DECAY_CONST[2]*tins))+ \
         self.QTERMS['Q32']*self.aonf[1]* \
         ((1-math.exp(-self.DECAY_CONST[1]*tins))/self.DECAY_CONST[1]- \
         (1-math.exp(-self.DECAY_CONST[2]*tins))/self.DECAY_CONST[2])+ \
         self.QTERMS['Q31']*self.QTERMS['Q21']*self.aonf[0]* \
         ((1-math.exp(-self.DECAY_CONST[0]*tins))/self.DECAY_CONST[0]- \
         (1-math.exp(-self.DECAY_CONST[2]*tins))/self.DECAY_CONST[2])+ \
         self.QTERMS['Q32']*self.QTERMS['Q21']*self.aonf[0]* \
         ((1-math.exp(-self.DECAY_CONST[2]*tins))/self.DECAY_CONST[2]- \
         (1-math.exp(-self.DECAY_CONST[1]*tins))/self.DECAY_CONST[1])
      c3*=self.beff[1]

      return (c2+c3)

   def CalcConc(self, cnts, flow, tins=600):
      """
      Returns calculated Rn-concentration [Bq/m3]
      cnts=NET COUNTS

      Revision history:
         2002-12-03
      """
      if tins==0: return 0.
      if tins!=self.ptins or not self.STerms:    # calculate STerms
         self.STerms={'S21':tins-self.QTERMS['Q21']/self.DECAY_CONST[0]*
            (1-math.exp(-self.DECAY_CONST[0]*tins))-self.QTERMS['Q12']/
            self.DECAY_CONST[1]*(1-math.exp(-self.DECAY_CONST[1]*tins)),

            'S22':tins-(1-math.exp(-self.DECAY_CONST[1]*tins))/
            self.DECAY_CONST[1],

            'S31':tins-self.QTERMS['Q21']*self.QTERMS['Q31']/self.DECAY_CONST[0]*
            (1-math.exp(-self.DECAY_CONST[0]*tins))-self.QTERMS['Q12']*
            self.QTERMS['Q32']/self.DECAY_CONST[1]*(1-
            math.exp(-self.DECAY_CONST[1]*tins))-
            self.QTERMS['Q13']*self.QTERMS['Q23']/self.DECAY_CONST[2]*
            (1-math.exp(-self.DECAY_CONST[2]*tins)),

            'S32':tins-self.QTERMS['Q32']/self.DECAY_CONST[1]*
            (1-math.exp(-self.DECAY_CONST[1]*tins))-self.QTERMS['Q23']/
            self.DECAY_CONST[2]*(1-math.exp(-self.DECAY_CONST[2]*tins)),

            'S33':tins-(1-math.exp(-self.DECAY_CONST[2]*tins))/
            self.DECAY_CONST[2]
         }
         self.ptins=tins

      if flow<5: return 0.


      try:
         C=cnts/(flow/3600.)/(self.beff[0]*(
         self.STerms['S21']/self.DECAY_CONST[0]+
         self.STerms['S22']/self.DECAY_CONST[1]
         ) + self.beff[1]*(
         self.STerms['S31']/self.DECAY_CONST[0]+
         self.STerms['S32']/self.DECAY_CONST[1]+
         self.STerms['S33']/self.DECAY_CONST[2])
         )
      except:
         print tins,cnts,flow
         print self.beff
         print self.STerms
         raise
#      print C
      return C


   def _Decay(self, tins=600):
      """
      Updates act on filter with decay (i.e. self.aonf)

      Revision history:
         2002-12-04 BUG FIX!! (was using updated self.aonf in calc.)
      """
      aonf=self.aonf[:]
      self.aonf[0]=aonf[0]*math.exp(-self.DECAY_CONST[0]*tins)
      self.aonf[1]=aonf[1]*math.exp(-self.DECAY_CONST[1]*tins)+ \
         self.QTERMS['Q21']*aonf[0]*(math.exp(-self.DECAY_CONST[0]* \
         tins)-math.exp(-self.DECAY_CONST[1]*tins))
      self.aonf[2]=aonf[2]*math.exp(-self.DECAY_CONST[2]*tins)+ \
         self.QTERMS['Q32']*aonf[1]*(math.exp(-self.DECAY_CONST[1]* \
         tins)-math.exp(-self.DECAY_CONST[2]*tins))+ \
         self.QTERMS['Q31']*self.QTERMS['Q21']*aonf[0]* \
         (math.exp(-self.DECAY_CONST[0]*tins)-math.exp(-self.DECAY_CONST[2]* \
         tins))+self.QTERMS['Q32']*self.QTERMS['Q21']*aonf[0]* \
         (math.exp(-self.DECAY_CONST[2]*tins)-math.exp(-self.DECAY_CONST[1]* \
         tins))

   def _AddAct(self, conc, flow, tins=600):
      """
      Adds activity to filter
      """
      c1=c2=c3=conc
      fl=flow/3600.
      vc1=c1*fl/self.DECAY_CONST[0]
      self.aonf[0]+=vc1*(1-math.exp(-self.DECAY_CONST[0]*tins))
      self.aonf[1]+=vc1*(1-self.QTERMS['Q21']*math.exp(-self.DECAY_CONST[0]* \
         tins)-self.QTERMS['Q12']*math.exp(-self.DECAY_CONST[1]*tins))+ \
         fl*c2/self.DECAY_CONST[1]*(1-math.exp(-self.DECAY_CONST[1]*tins))
      self.aonf[2]+=vc1*(1-self.QTERMS['Q21']*self.QTERMS['Q31']* \
         math.exp(-self.DECAY_CONST[0]*tins)-self.QTERMS['Q12']* \
         self.QTERMS['Q32']*math.exp(-self.DECAY_CONST[1]*tins)- \
         self.QTERMS['Q13']*self.QTERMS['Q23']*math.exp( \
         -self.DECAY_CONST[2]*tins))+ \
         fl*c2/self.DECAY_CONST[1]*(1-self.QTERMS['Q32']*math.exp( \
         -self.DECAY_CONST[1]*tins)-self.QTERMS['Q23']*math.exp( \
         -self.DECAY_CONST[2]*tins))+ \
         fl*c3/self.DECAY_CONST[2]*(1-math.exp(-self.DECAY_CONST[2]*tins))


   def _IterateStartConc(self, ar, secsSinceFChange, bg=DEFBGCNTS):
      """
      Find a starting concentration value for C (and aonf) by
      iteration. Returns concentration and updates aonf[]
      """
      nCnts=ar[0][1]-bg
      flow=ar[0][2]
      climit=math.sqrt(nCnts)
      secS=secsSinceFChange
      if secS>14400: secS=14400
      dts=(ar[1][0]-ar[0][0]).seconds
      iCnts=nCnts/8           # initial coll count guess
      for n in xrange(999):
         C=self.CalcConc(iCnts, flow, dts)
         self.aonf=[0.,0.,0.]
         self._AddAct(C, flow, secS)
         decCnts=self.DecayCounts(dts)
         tCnts=decCnts+iCnts
         cdiff=nCnts-tCnts
         if abs(cdiff)<climit: break
         if cdiff<0:
            iCnts-=iCnts*.09
         else:
            iCnts+=iCnts*.1
      else:
         raise "No luck in iteration..."
      return C

   def _FindFCTime(self, C, el, bg):
      """
      Finds the 'exact' time for the filter change (or collection
      start) from C,counts. Updates aonf[].

      NOTE: fixed 600 s period lenght
      """
      nCnts=el[1]-bg
      if nCnts<10: nCnts=10
      minC=999; mint=0
      for t in xrange(120,1200,10):
         cnts=nCnts
         if t>600:
            self.aonf=[0.,0.,0.]
            self._AddAct(C,el[2],t-600)
            cnts=nCnts-self.DecayCounts(600)
            if cnts<1: cnts=1
         Cx=self.CalcConc(cnts, el[2], t)
         dC=abs(Cx-C)
         if dC<minC: minC=dC; mint=t
      self.aonf=[0.,0.,0.]
      self._AddAct(C,el[2],mint)
      return mint

   def _DataRestart(self, ar, ind, dts, bg):
      """
      Find a 'suitable' value for concentration after missing period.
      This is done from net counts detected in first period
      after a gap.
      """
      self._Decay(dts-600)     # decay
      dCnts=self.DecayCounts(600)
      nc=ar[ind][1]-bg-dCnts
      if nc<10: return
      climit=math.sqrt(nc)
      paonf=self.aonf[:]
      i=1
      while i<900:
         a=i*.5
         self.aonf=paonf[:]
         self.aonf[0]+=a/3.      # factor experimentally
         self.aonf[1]+=a/1.1     # defined, ca 10+ min
         self.aonf[2]+=a
         C1=self._GetC(600,ar[ind+1],0,bg)
         C2=self._GetC(600,ar[ind+2],0,bg)
         if abs(C2-C1)<.5: break
         i+=1
      if i>=900:
         raise "No AonF values found..."
      self.aonf=paonf[:]
      a=i*.5
      self.aonf[0]+=a/3.; self.aonf[1]+=a/1.1; self.aonf[2]+=a


   def _GetC(self,dts,el,Cp,bg):
      """
      Return calc C. Updates also A on filter
      """
      dCnts=self.DecayCounts(dts)
      nCnts=el[1]-bg-dCnts

#      print "t,Cprev,decCnts,netCnts:",dts,Cp,dCnts,nCnts,
#      print "aonf:",self.aonf

      if nCnts<2: nCnts=2
      C=Cp
      if el[2]>1:
         C=self.CalcConc(nCnts,el[2],dts)
         if Cp>0: C=(C+Cp)/2
         if C<.02: C=.02
      self._Decay(dts)
      if el[2]>1:
         self._AddAct(C,el[2],dts)
      return C

   def _WriteNote2Db(self,db,tblname,flag,el,txt,dt2=None):
      """
      Writes to note db tbl
      """
      if (not db) or (not tblname): return
      dbcu=db.cursor()
      dt=el[0]
      strng=txt+":%d,%.1f" % (el[1],el[2])
      if dt2:
         dt2s="'%s'" % dt
         dt=dt2
      else:
         dt2s="NULL"
      s="INSERT INTO %s VALUES ('%s',%s,' ','%s','%s');" % \
         (tblname, dt, dt2s, flag, strng)

      try:
         dbcu.execute("DELETE FROM %s WHERE datetime='%s';" % (tblname, dt))
         db.commit()
      except:
         pass

      dbcu.execute(s)
      dbcu.close()
      db.commit()



   def GetRnConc(self, ar, bg=DEFBGCNTS, db=None, tblname='_rn_notes'):
      """
      Return calculated 10-min Rn-concentrations as an array

      ar: array [DateTime dt, int cnts, float flow]
      bg: background counts in 10-min period
      """
      resar=[]
      ind=0; C=0.;
      dts=(ar[1][0]-ar[0][0]).seconds
      if self.dtp.ticks()==0: self.dtp=ar[0][0]
      fchange=0
      debug=0
      noflow=0
      noflowstart=DateTime.localtime(0)

      for el in ar:
         if (el[0].minute % 10) !=0:
            print "Wrong time:",el
            self._WriteNote2Db(db,tblname,'T',el,'')
            ind+=1
            continue  # some errors in data
         if ind==0 and self.Cp==0 and el[1]>500:
            C=self._IterateStartConc(ar[:2], 9999, bg)
         else:
            dts=(el[0]-self.dtp).seconds
            if ind>0:
               if el[2]<ar[ind-1][2]-2:
                  if ind+1>=len(ar): return resar
                  cnxt=ar[ind+1][1]
                  if cnxt<500 and cnxt<el[1]-50 and \
                  (ar[ind+1][0]-el[0]).minutes<12:
                     fchange=1
                     print "Filter change(?) at", el[0]
                     self._WriteNote2Db(db,tblname,'R',el,'?')
                  else:
                     if el[2]<15:
                        self._WriteNote2Db(db,tblname,'W',el,'')

            if el[2]<2 and noflowstart.ticks()==0:
               noflowstart=el[0]

            if el[2]>=2 and noflowstart.ticks()>0:
               if (el[0]-noflowstart).seconds>7200:
                  noflow=1
                  print "No flow: %s - %s" % (noflowstart, el[0])
                  self._WriteNote2Db(db,tblname,'W',el,'',noflowstart)
                  dts=(el[0]-noflowstart).seconds
               noflowstart=DateTime.localtime(0)

            if dts>660 or noflow:
               if not noflow:
                  print "Missing data: %s, difft: %.1f h" % (el[0],dts/3600.)
                  self._WriteNote2Db(db,tblname,'D',el,'',self.dtp)
               self._DataRestart(ar, ind, dts, bg)
               self.Cp=0
               noflow=0
               fchange=3      # over rest of the calculations

            if fchange==2:
               self._FindFCTime(C,el,bg)  # updates vars
               fchange+=1
            elif fchange==0:
               C=self._GetC(dts,el,self.Cp,bg)
            else:
               fchange+=1

#         if el[0].month==8 and el[0].day==12: debug=1
#         else: debug=0
         if debug:
            print ind,":",el[0],el[1],el[2],fchange,C,":",self.aonf[2]
         self.dtp=el[0]
         if fchange==0 and el[2]>1 and C>0:
            resar.append([el[0],C,el[1],el[2]])
            self.Cp=C
         elif fchange>=3:
            fchange=0
         ind+=1

      return resar






