# -*- coding: latin-1 -*-
"""
   Common/general time related python routines.
   
   Revision history:
      2007-03-26  +ToDST()
      2013-12-04  +TZStr()
"""
import mx.DateTime as DateTime
import datetime, time

NOW=DateTime.now
DTDT=DateTime.DateTime
TDELTA=DateTime.TimeDelta
TYPEDT=type(NOW())

def FirstStartSaveDt(DT=None,dmins=1):
   """
   Return [start,first] save DateTime after time DT (None --> use current),
   when save interval is dmins, so that start  is divisable by full hour
   (if possible).
   
   Revision history
   """
   dt=DT
   if not dt: dt=NOW()
   assert dmins>0
   dmn=int(dmins*10)
   dtcm=DTDT(dt.year,dt.month,dt.day,dt.hour,dt.minute)
   if dmn not in [10,15,20,25,30,40,50,60,75,100,120,150,200,300,600]:
      dtstart=dtcm+TDELTA(0,1)
   else:
      dtm=TDELTA(0,dmins)
      dtstart=dtcm-TDELTA(0,dtcm.minute)
      while dtstart<dt: 
         dtstart+=dtm
   dtf=dtstart+TDELTA(0,dmins)
   return [dtstart,dtf]

   
def ToDST(dt):
   '''
      Return dt converted to DST (European convention),
      ASSUMES, that dt is non-DST time. That is, adds
      one hour if dt within DST period, otherwise returns dt
      
      DST starts at last Sunday in March at 03 (-> 04)
      and ends at last Sunday in October at 04 (-> 03)
      
      
      Revision history:
         2007-03-26  1st version
   '''
   if not type(dt)==TYPEDT:
      mx=False
      dh=datetime.timedelta(seconds=3600)
   else:
      mx=True
      dh=TDELTA(1)

   mo=dt.month
   if 3<mo<10: return dt+dh
   if mo in [1,2,11,12]: return dt
   
   dtmx=DTDT(dt.year,dt.month,dt.day,dt.hour)
   d=DTDT(dtmx.year,mo,dtmx.days_in_month)
   while d.day_of_week!=6: d-=1
   dd=DTDT(d.year,mo,d.day,3)
   if (mo==3 and dtmx>=dd) or (mo==10 and dtmx<dd): return dt+dh
   return dt
   
def TZStr():
   '''
      Return current system tz as a string:
      UTC+-h (if in UTC: UTC+0)
   '''
   is_dst=time.daylight and time.localtime().tm_isdst>0
   utc_offset = -(time.altzone if is_dst else time.timezone)/3600
   return 'UTC%+d'%(utc_offset,)
