# -*- coding: utf-8 -*-
"""
   Various utilities for simple html generation
   
      2014-11-04  DOCTYPE to html, Header: +javascript
      2015-02-19  GenSelect: set also id to name
"""
import mx.DateTime as DateTime
from sgmllib import SGMLParser
import urllib,os
from fnmatch import fnmatch

HTML_SPECIAL_CHARS={'&':'&amp;','<':'&lt;','>':'&gt;',
   '"':'&#34;'}

class cGenHTML:
   '''
      HTML generation helpers
   '''

   def __init__(self):
      '''
      '''
      pass


   def EscapeChars(self,s,alsoHTMLChars=False):
      """
      Return s with special characters encoded to HTML standard.

      Revision history:
         2007-11-17/18  more characters, alsoHTMLChars
      """
      if not s: return s
      try:
         slist=list(s)
      except:
         print s
         raise
      for i in range(len(slist)):
         ordin=ord(slist[i])
         if 128 < ordin < 256: slist[i] = "&#%d;"%ordin
         if alsoHTMLChars and slist[i] in HTML_SPECIAL_CHARS:
            slist[i]=HTML_SPECIAL_CHARS[slist[i]]
      return ''.join(slist)


   def Header(self,title="Title",toHead="",hdr2title=0,noCache=True,javascript=''):
      """
      Return standard HTML-head with title (default='Title') and optional
      other data (e.g. JavaScript) to the head, ending with <body>.
      If hdr2title is 1-5 also adds <h?>title</h?> after body.
      """
      s=[]
      s.append('''<!DOCTYPE html>''')
      s.append("<html><head>")
      s.append("<title>%s</title>"%title)
      s.append('<meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1">')
      if noCache:
         s.append('<meta http-equiv="Expires" content="0">')
         s.append('<meta http-equiv="Pragma" content="no-cache">')
         s.append('<meta http-equiv="Cache-Control" content="no-cache,no-store,must-revalidate,private">')
      if toHead:
         s.append(toHead)
      if javascript :
         s.append('<script type="text/javascript" %s></script>'%(javascript,))
      s.append('</head><body>')
      if 0 < hdr2title < 6:
         s.append("<h%d>%s</h%d>"%(hdr2title,title,hdr2title))
      return "\n".join(s)


   def Footer(self,back="",addtogen=""):
      """
      line, creation time, </body></html>
      """
      s="<hr>"
      if back:
         s+='<a href="%s">back to previous</a><br>\n'%back
      s+="Page generated at %19.19s."%(DateTime.now(),)
      if addtogen:
         s+=" %s"%addtogen
      return s+'</body></html>\n'

   def Table(self,data,hdng=[],border=1,cellpadding=1,
      cellspacing=1, center=False,right=False,width="100%"):
      """
      Return data[[]] as HTML-table with headers hdng[]
      """
      s=[]
      c1=''
      if center:
         c1=' ALIGN="CENTER"'
      elif right:
         c1=' ALIGN="RIGHT"'
         
      swidth=''
      if width:   sw='width="%s"'%swidth
      s.append('<TABLE border=%d cellpadding=%d cellspacing=%d %s>'%(
         border,cellpadding,cellspacing,swidth))
      if hdng:
         s.append('<THEAD><TR>')
         for hd in hdng: s.append('<TH>%s</TH>'%self.EscapeChars(hd))
         s.append('</TR></THEAD>')
      s.append('<TBODY>')
      for ln in data:
         s.append('<TR>')
         for cl in ln:
            s.append('<TD%s>%s</TD>'%(c1,cl))
         s.append('</TR>')
      s.append('</TBODY></TABLE>')
      return '\n'.join(s)


   def GenSelect(self,name,optionlist,selected=None,title="",size="1",onChange=None):
      '''
         Generate <SELECT><OPTION...>...</OPTION></SELECT>

         Revision history:
            2007-02-12 str(o)==str(selected)
            2007-08-24 onChange added
      '''

      ses='<SELECT NAME="%s" SIZE="%s" TITLE="%s" id="%s"'%(name,size,title,name)
      if onChange:
         ses+=' onChange="%s"'%onChange
      ses+='>'
      s=[ses]
      for o in optionlist:
         sel="SELECTED" if str(o)==str(selected) else ""
         s.append("<OPTION %s>%s</OPTION>"%(sel,o))
      s.append('</SELECT>')
      return '\n'.join(s)

   def GenCheckbox(self,list,selected=[],title=""):
      '''
      '''
      s=[]
      for nd in list:
         sel="" if nd not in selected else "CHECKED"
         s.append('<INPUT TYPE="checkbox" NAME="%s" TITLE="%s" %s>%s</INPUT>'%(nd,title,sel,nd))
      return '\n'.join(s)



class cURLLib:
   '''
      Different URL utilities for down/uploading files etc.

      Revision history:
         2007-11-18 start of devel
   '''
   class URLLister(SGMLParser):
      '''
         helper class
      '''
      def reset(self):
         SGMLParser.reset(self)
         self.urls = []
      def start_a(self, attrs):
         href = [v for k, v in attrs if k=='href']
         if href:  self.urls.extend(href)

   def __init__(self):
      '''
      '''
      pass

   def GetFilenamesOnServer(self,url,pattern='*'):
      '''
         Return files (not dirs or others) found in url
         as {basefile:[version,wholename]}
      '''
      assert url[:6]=='ftp://' or url[:7]=='http://'
      ret=[]
      f=urllib.urlopen(url)
      content=f.read()
      f.close()
      if url[:7]=='http://':
         parser=self.URLLister()
         parser.feed(content)
         parser.close()
         for u in parser.urls:
            ret.append(u)
      else:    # ftp, just strings, pick last one as file name
         pats=[pattern]  if type(pattern)==type('') else pattern
         for ln in content.split('\n'):
            s=ln.split()
            if len(s)<2: continue
            for pat in pats:
               fi=os.path.basename(s[-1])
               if fnmatch(fi,pat):
                  ret.append(fi)
                  break
      return ret

   def RetrieveFile(self,url,resdir=''):
      '''
         Retrieve file using urllib and save it to resdir
         Return name of the result file
      '''
      dir,base=os.path.split(url)
      ofile=os.path.join(resdir,base)
      f=urllib.urlopen(url)
      ofs=open(ofile,'wb')
      ofs.write(f.read())
      ofs.close()
      f.close()
      return ofile


if __name__ == "__main__":
   # TESTS...
   u=cURLLib()
   url='ftp://ftp.funet.fi/pub/mirrors/ftp.gnu.org/pub/gnu/autoconf'
#   fls=u.GetFilenamesOnServer(url,pattern='autoconf-*.tar.*')
   url+='/autoconf-2.61.tar.bz2'
   u.RetrieveFile(url)


