'''
   Misc utilities for handling data from pallas
   
   Devel start 2006-04-07
   
   Revision history
      2015-01-23  +GetSammalPowerOutages, psycopg2->cDB
      2015-07-03  inutc= addition to GetSammalPowerOutages
'''
import os
import mx.DateTime as DateTime
from jahdblib import cDB

DTDT=DateTime.DateTime
TDELTA=DateTime.TimeDelta

def GetSammalVisitTimes(startdt=DTDT(1990,1,1), enddt=DateTime.now()):
   """
      Return visit times in Sammaltunturi as a list of tuples: 
         [(dts,dte)]
      
      If visit end time is missing, assumes 1 h visit
   """
   db=cDB("pallas_meta")
   cu=db.cursor()
   s="""
      SELECT visit_start_dt,visit_end_dt FROM sammal_log
      WHERE visit_start_dt>='%s' AND visit_start_dt<'%s'
      ORDER BY visit_start_dt;
   """ % (startdt, enddt)
   cu.execute(s)
   da=cu.fetchall()
   db.close()
   res=[]
   for d in da:
      edt=d[1]
      if not edt or edt<=d[0]: 
         edt=d[0]+DateTime.TimeDelta(1)
      res.append((d[0],edt))
   return res
   
def GetSammalPowerOutages(startdt=DTDT(1998,7,1),enddt=DateTime.now(),inutc=False):
   """
      Return periods when sample diff.p < 2 AND vacuum line diff.p<500
      between startdt,enddt as [[startdt,enddt]]
      
      Assumes UTC+2
   """
   toutc=TDELTA(-2) if inutc else TDELTA(0)
   db=cDB("pallas_meta")
   cu=db.cursor()
   sq="""
   SELECT datetime FROM sammal_diffpress WHERE datetime>='%s' AND datetime<'%s' 
   AND (dp_sampleline_hpa IS NOT NULL) AND dp_sampleline_hpa<2 AND
   dp_vacuumline_hpa<500 ORDER BY datetime;"""%(startdt-1,enddt+1)
   cu.execute(sq)
   if cu.rowcount==0: return []
   da=cu.fetchall()
   ret=[]
   idx=0
   lenda=len(da)
   sdt=edt=None
   while idx<lenda:
      if not sdt: 
         sdt=edt=da[idx][0]+toutc
      else:
         if (da[idx][0]-da[idx-1][0]).minutes>2:
            ret.append([sdt,edt])
            sdt=edt=da[idx][0]+toutc
         else:
            edt=da[idx][0]+toutc
      idx+=1
   return ret

