'''
   Fit quadratic curve to x,y,sx,sy data set using
   ODR, weighted orthogonal distance regression.
   sx,sy are deviations for the x,y values.

   Revision history:
      2007-10-05 primitive model='cubic' added
      2008-02-29 only o.stopreason[-1] checked
      2008-04-22 no exception for stopreason "Iteration limit reached"
      2009-09-29 use scipy.odr
      2009-10-13 stop reason check removed: it was from time-to-time non-existent,
        and not consistent
      2011-09-28 'quadratic_c1' added (factor c forced to 1)
'''
from numpy import *
from scipy.odr import *


def _cubic_est(data):
   return (1.,1.,1.,1.)
def _cubic(B,x):
   return x*(x*(x*B[0]+B[1])+B[2])+B[3]
cubic=Model(_cubic,estimate=_cubic_est)

def _quadratic_est(data):
   return (1.,1.,1.)
def _quadratic(B,x):
   return x*(x*B[0]+B[1])+B[2]
quadratic=Model(_quadratic,estimate=_quadratic_est)

def _quadratic_c1(B,x):
   return x*(x*B[0]+B[1])+1.
quadratic_c1=Model(_quadratic_c1,estimate=_quadratic_est)

def _unilinear_est(data):
   return (1.,1.)
def _unilinear(B,x):
   return x*B[0]+B[1]
unilinear=Model(_unilinear,estimate=_unilinear_est)


def Fit(x,y,sx,sy,model='quadratic',printtoscr=False):
   '''
      Do the fit, return fitted factors ('beta') and residual
      variance as a ([beta],res_var).

      Equation: y==list[0]*x*x+list[1]*x+list[2] OR
      if model='unilinear' ...
      Print out the results if printtoscr==True.

      x,y,sx,sy can be arrays or lists, and sx,sy also constants,
      if all the deviations are the same.
   '''
   dat=RealData(array(x),array(y),sx=array(sx),sy=array(sy))
   model=eval(model)
   myodr=ODR(dat,model)
   myodr.set_job(fit_type=0)
   o=myodr.run()
   if printtoscr:
      o.pprint()
#   print "o.stopreason:",o.stopreason
#   if o.stopreason[-1] not in \
#      ["Sum of squares convergence",
#      "Both sum of squares and parameter convergence",
#      "Parameter convergence",
#      "Iteration limit reached"]:
#      raise Exception("Problems with ODR, stop reason:'%s'?"%str(o.stopreason))
   return (list(o.beta),o.res_var)



if __name__ == '__main__':
   print "\n\tquad_odr.py TEST\n"
   y=[340.2,324.,316.4,297.5,282.9]
   x=[.943947,.986163,1.00342,1.05889,1.10642]
#   sy=.00001 ; sx=.00001
   sy=[1.2,1.1,.8,.7,.9]
   sx=.005
   print "x:%s\n  sx:%s"%(x,sx)
   print "y:%s\n  sy:%s"%(y,sy)
   print
   print "\nQUADRATIC:"
   b=Fit(x,y,sx,sy,printtoscr=True)
   for i,xx in enumerate(x):
      v=b[0][0]*xx*xx+b[0][1]*xx+b[0][2]
      print "%.5f  %.2f  %.2f  %.2f"%(xx,y[i],v,y[i]-v)
   print "\nUNILINEAR:"
   b=Fit(x,y,sx,sy,model='unilinear',printtoscr=True)
   for i,xx in enumerate(x):
      v=b[0][0]*xx+b[0][1]
      print "%.5f  %.2f  %.2f  %.2f"%(xx,y[i],v,y[i]-v)

